﻿* Encoding: UTF-8.

****************************************************
CFSR 3: CHILD POPULATIONS BY STATE
****************************************************

* This syntax is being provided for those interested in learning how CB obtains child population data, and for 
* building states' capacity to produce these data moving forward. Child population estimates are used to 
* calculate states' entry rates. Entry rate is used as a risk adjustment variable for two indicators: Permanency 
* in 12 months for children entering foster care and Re-entry to foster care in 12 months. For details on how entry 
* rates are calculated, see the Federal Register notice and the relevant syntax used to calculate observed 
* performance for the two indicators. 

* Note: A file containing child population estimates for years 2011, 2012, 2013, 2014, 2015, and 2016 is already available in 
* the CFSR3IndicatorSyntax.zip file. When that file is unzipped, it will be saved at 
* the Fixed files directory. The file includes child population estimates for all 50 states, the 
* District of Columbia and Puerto Rico.These historical estimates are based on Census 
* estimates with a release date of June 2017, using the data available form the Census website.
* CB selected only records where AGE was less than 18, SEX was equal to 0 (0 = males and females), and ORIGIN was equal to 0 (0 = 
* all origins), and then summed the population estimates for each state, for each year available in the file.

****************************************************

* Population estimates for years 2017 and beyond can be processed using the syntax below, which will also 
* append each new year of data to the existing Child populations.sav file. The syntax in this example uses the 
* 2016 census year as an example, even though at the time this syntax is written that file is not yet available.  
*The set of macros at the beginning may need to be changed to match the year being 
* processed. 

* 1.	Go to http://www.census.gov/popest/data/index.html
* 2.	For States (Level of Geography), Population by age, sex, race, and Hispanic origin (Level of Detail), click the 
*     link under Most Current Data (e.g., V2016)
* 3.	Under Downloadable Datasets, click the link for Population estimates by age (18+), select the CSV link and save the data to 
*     your computer (e.g., C:\CFSR3\) as a CSV file. This file (e.g., SCPRC-EST2016-18+POP-RES.csv) contains 
*     population data for all 50 states, D.C., and Puerto Rico. 



*In all four macros, change the year to reflect the Census year shown in the file you downloaded.
define YY () 16 
!enddefine.
define YYYY () 2016 
!enddefine.
define POPESTIMATE () POPESTIMATE2016
!enddefine.
define POPEST18PLUS () POPEST18PLUS2016
!enddefine.


* Root directory.
file handle Root_FH /name="C:\CFSR 3".

* Directory for the Census data file.
file handle Census_FH /name="Root_FH\0 - Source Data\Census".

* Folder where child population files are located (fixed files are generally useful static files that are used in various different sets of code).
file handle Fixed_FH /name="Root_FH\1 - Fixed files".

GET DATA  /TYPE=TXT
  /FILE= 'Census_FH\SCPRC-EST2016-18+POP-RES.csv'
  /DELCASE=LINE
  /DELIMITERS=","
  /ARRANGEMENT=DELIMITED
  /FIRSTCASE=2
  /IMPORTCASE=ALL
  /VARIABLES=
  SUMLEV F3.0
  REGION A1
  DIVISION A1
  STATE F2.0
  NAME A24
  POPESTIMATE2016 F9.0
  POPEST18PLUS2016 F9.0
  PCNT_POPEST18PLUS F4.1.
CACHE.
EXECUTE.
DATASET NAME CurrentYear WINDOW=FRONT.

compute ReleaseDate = date.dmy(01,06,YY+1).
formats ReleaseDate (adate10).
execute.

* 6.	Subtract POPEST18PLUS from POPESTIMATE. This yields the child populations (17 and under) for all 50 states, D.C., and Puerto Rico. 

compute ChildPopulation = POPESTIMATE - POPEST18PLUS.
execute.

* 7.	Delete the record where STATE = 0, which holds the total population of the United States.

select if STATE <> 0.
execute.

* 8. Create variable holding the census year the population estimate is based on (e.g., July 1st, YYYY). 
*     Census estimates are always based on July 1st of the given year.

compute PopYear = date.dmy(01, 07, YYYY).
formats PopYear (adate10).
execute.

* 9.	Append the new year's estimates for this release to the existing Child populations.sav file.

get file 'Fixed_FH\Child populations.sav'.
dataset name PastYears window=front.
dataset activate PastYears.
add files /FILE=*
  /FILE='CurrentYear'
  /RENAME (DIVISION NAME PCNT_POPEST18PLUS POPEST18PLUS POPESTIMATE REGION SUMLEV=d0 d1 d2 
    d3 d4 d5 d6)
  /DROP=d0 d1 d2 d3 d4 d5 d6.
execute.
dataset close CurrentYear.
sort cases by STATE PopYear.
save outfile 'Fixed_FH\Child populations.sav'.



