﻿* Encoding: UTF-8.
set printback=on.


****************************************************
* MERGE 6-MONTH SUBMISSIONS
****************************************************

* Calculating performance on CFSR 3 indicators requires multiple 6-month submissions. The number of 
* submissions needed depends on the indicator. The date of those submissions (i.e., the reporting 
* period) depends on the the 12-month cohort for which performance will be measured. For example, to 
* measure performance for Permanency in 12 months for children entering foster care (and the companion 
* Re-entry indicator) requires six, 6-month submissions (i.e., three years of data). So, to measure Permanency
* for children who entered during 11B12A requires these 6-month submissions: 11B, 12A, 12B, 13A, 13B, and 
* 14A. 

* This step will merge together all 6-month submissions stored in the 'AFCARS Submissions' folder. In determining 
* what 6-month files to include, CB recommends starting with 09B and ending with the most recent submission 
* available (i.e., from the latest reporting period). In other words, CB recommends building and maintaining 
* a historical dataset starting with 09B (although there is no problem with adding earlier periods). 

* Merge together all files in the submissions folder.
****************************************************

* Root directory. ****PLEASE CHANGE THIS BASED ON WHERE YOU UNZIP AND SAVE THE CFSR 3 FOLDER.****.
file handle Root_FH /name="C:\CFSR 3".

***DO NOT CHANGE FILE HANDLES BELOW****.
*Directory to save combined AFCARS file.
file handle AFCARS_Comb_FH/name='Root_FH\0 - Source Data'.

*Directory for all AFCARS Submissions.
file handle AFCARS_Source_FH/name='AFCARS_Comb_FH\Submissions AFCARS'.

* Folder where fixed files are located (fixed files are generally useful static files that are used in various different sets of code).
file handle Fixed_FH /name="Root_FH\1 - Fixed files".

* Merge files. This syntax requires that SPSS Python Essentials be installed. It is installed by default 
* beginning with SPSS Version 22.  

begin program.
import spss, spssaux, os, glob

fh = spssaux.FileHandles()

rdir = fh.resolve('AFCARS_Source_FH')  # Specifies folder containing .sav files.
fils = sorted([fil for fil in os.listdir(rdir) if fil.endswith('.sav')])
spss.Submit('get file "%s/%s".'%(rdir,fils.pop(0)))
for rep in range(len(fils)/49 + 1):
    spss.Submit('add files file=*/%s.'%'/'.join(['file="%s"'%os.path.join(rdir,fil) for fil in fils[49*rep:49*rep + min(49,len(fils)-49*rep)]]))
spss.Submit('exe.')
end program.

dataset name SourceFile.
Cache.
EXECUTE.
****************************************************
* FILE PREP
****************************************************

* Create a unique, numeric ID (Child ID) across the entire file.
****************************************************

* Create a unique ID (recnumbr2) across the entire file (state FIPS + recnumber).
sort cases by state recnumbr.
string statestring (A2). 
compute statestring = STRING(state, F2.0).

string recnumbr2 (A50).
compute recnumbr2=concat(statestring,recnumbr).


* Create a unique, numeric version of recnumbr2, called ChildID. ChildID will be used later for sorting, breaking, 
* and other functions. It is more efficient and reliable than using the recnumbr2, which is a string and, for some 
* states, contains highly specialized characters.  
autorecode VARIABLES=recnumbr2 
   /INTO ChildID.
value labels ChildID.
delete variables statestring recnumbr2.

* Add state names and abbreviations.
****************************************************

* Add state full names and abbreviations by merging these in from states.sav data file. These are useful for 
* reporting purposes, custom tables, etc.
get file 'Fixed_FH\states.sav'.
dataset name states window=front.
sort cases by state.
save outfile='Fixed_FH\states.sav'.
dataset activate SourceFile.
dataset close states.
match files /FILE=*
  /TABLE='Fixed_FH\states.sav'
  /BY state.
execute.



* Create date fields.
****************************************************

* Start and end dates for each 6-month reporting period in the merged file.
* ... A files (Oct 1 - Mar 31).
if repdatmo = 3 DtReportBeg = date.mdy(10,1,repdatyr-1).
if repdatmo = 3 DtReportEnd= date.mdy(3,31,repdatyr).
* ... B files (Apr 1 - Sep 30).
if repdatmo = 9 DtReportBeg = date.mdy(4,1,repdatyr).
if repdatmo = 9 DtReportEnd = date.mdy(9,30,repdatyr).

* Date of the last 6-month reporting period in the merged file.
aggregate 
  /OUTFILE=* MODE=ADDVARIABLES
  /BREAK=
  /DtReportEndFinal=MAX(DtReportEnd).

* Other key dates. Some of these are not used in the CFSR syntax, and are created for convenience in case 
* users are interested in making use of them).
compute DtReview=date.mdy(pedrevmo, pedrevda,pedrevyr).
compute DtBirth=date.mdy(dobmo,dobda,dobyr).
compute DtFirRem=date.mdy(rem1mo,rem1da,rem1yr).
compute DtPriorDisch =date.mdy(dlstfcmo,dlstfcda,dlstfcyr).
compute DtLatRem=date.mdy(latremmo,latremda,latremyr).
compute DtLatRemTrans=date.mdy(remtrnmo, remtrnda, remtrnyr).
compute DtCurSet=date.mdy(cursetmo,cursetda,cursetyr).
compute DtTPRMom=date.mdy(prtmommo, prtmomda, prtmomyr).
compute DtTPRDad=date.mdy(prtdadmo, prtdadda, prtdadyr).
compute DtDisch=date.mdy(dodfcmo,dodfcda,dodfcyr).
compute DtDischTrans=date.mdy(dodtrnmo,dodtrnda,dodtrnyr).
execute.

* Formatting.
variable labels 
DtReportBeg 'Start date of the 6-month reporting period (10/1/YYYY or 4/1/YYYY)'
DtReportEnd 'End date of the 6-month reporting period (3/31/YYYY or 9/30/YYYY)'
DtReportEndFinal 'Date of the last 6-month reporting period in the file'
DtReview 'Date of most recent periodic review'
DtBirth 'Date of birth'
DtFirRem 'Date of first removal from home'
DtPriorDisch 'Date of discharge from last foster care episode' 
DtLatRem 'Date of latest removal from home'
DtLatRemTrans 'Date (transaction) of latest removal from home'
DtCurSet 'Date of placement in current foster care setting'
DtTPRMom 'Date of mother parental rights termination'
DtTPRDad 'Date of father parental rights termination'
DtDisch 'Date of discharge from foster care' 
DtDischTrans 'Date (transaction) of discharge from foster care'.
formats DtReportBeg to DtDischTrans (adate10).

* Delete original date fields.
delete variables
repdatyr repdatmo
pedrevyr pedrevmo pedrevda 
dobyr dobmo dobda 
rem1yr rem1mo rem1da 
dlstfcyr dlstfcmo dlstfcda 
latremyr latremmo latremda 
remtrnyr remtrnmo remtrnda 
cursetyr cursetmo cursetda 
prtmomyr prtmommo prtmomda 
prtdadyr prtdadmo prtdadda 
dodfcyr dodfcmo dodfcda 
dodtrnyr dodtrnmo dodtrnda. 

* Delete duplicate records.
****************************************************

* Identify children reported two or more times during the same 6-month reporting period. These occur rarely, as 
* states are advised to report only the most recent episode per 6-month reporting period.
SORT CASES BY ChildID(A) DtReportBeg(A) DtLatRem(A) DtDisch(A).
MATCH FILES
  /FILE=*
  /BY ChildID DtReportBeg
  /FIRST=PrimaryFirst
  /LAST=PrimaryLast.
DO IF (PrimaryFirst).
COMPUTE  MatchSequence=1-PrimaryLast.
ELSE.
COMPUTE  MatchSequence=MatchSequence+1.
END IF.
LEAVE  MatchSequence.
FORMATS  MatchSequence (f7).
COMPUTE  InDupGrp=MatchSequence>0.
SORT CASES InDupGrp(D).
MATCH FILES
  /FILE=*
  /DROP=PrimaryFirst MatchSequence.
VARIABLE LABELS  PrimaryLast 'Indicator of each last matching case as Primary'.
VALUE LABELS  PrimaryLast 0 'Duplicate Case' 1 'Primary Case'.
VARIABLE LEVEL  PrimaryLast (ORDINAL).
EXECUTE.

* Document states that reported the same children two or more times in a report period, and the number of 
* duplicates reported.
temporary. 
select if InDupGrp = 1 and PrimaryLast = 0.
EXECUTE.

* Discard duplicates. The sort order means we try to keep the record with the latest removal date (if it differs) 
* and, among those, the record with a discharge date (if present).
select if PrimaryLast = 1.
execute.

delete variables PrimaryLast InDupGrp.

*Add rptyr and rptmo variables needed for later merge with NCANDS data.

compute rptyr = xdate.year(dtreportbeg).
compute rptmo = xdate.month(dtreportbeg).
execute.

if (rptmo eq 10) FY eq rptyr + 1.
if (rptmo eq 4) FY eq rptyr.
execute.

string afcarsid (a36).
compute afcarsid= recnumbr.
execute.

delete variables rptyr rptmo.


* Delete records with no afcarsid.
****************************************************

SELECT IF AFCARSID <> ''.
EXECUTE.

****************************************************
* PREPARATION FOR DATA QUALITY - AFCARS CROSS-FILE CHECKS
****************************************************

* Cross-file checks look across every two consecutive 6-month reporting periods in the merged file to determine
* if a child was reported in the next 6-month report period. Two checks are then carried out:

* 1. Dropped records
* 2. AFCARS IDs don't match from one period to the next

* These checks are not performed on records that occur in the last reporting period in the merged file, because
* there is no subsequent 6-month reporting period to look for.

* Preparation - Calculate TimeBetweenReports.
****************************************************

* First, calculate the time in months between the child's record and any subsequent report that may exist for 
* the child. The two DQ checks will use this information. 
sort cases BY ChildID(A) DtReportBeg(A).

* For each record, make a copy of DtReportBeg, which stores the start date of the 6-month reporting period.
compute DtReportBeg1 = DtReportBeg.
execute.
formats DtReportBeg1 (adate10).

* Create a variable called Next6MoReport. If the child was reported in a subsequent 6-month period (i.e., 
* another DtReportBeg for this child is found), copy the start date of that subsequent 6-month submission 
* into Next6MoReport. If the child was not reported in a subsequent submission, leave Next6MoReport blank. 
* Note: For a new child, DtReportBeg1 is set to missing; otherwise, the start date will will get assigned to the 
* previous child. 
PRESERVE.
set workspace 200000.
if (ChildID<>lag(ChildID)) DtReportBeg1 = $sysmis.
create Next6MoReport=Lead(DtReportBeg1,1).
execute.
formats Next6MoReport (adate10).
RESTORE.


* Calculate the number of months between the start date of the current 6-month reporting period and the start 
* date of the subsequent 6-month report period that was found for this child. If the reporting period is the last
* reporting period in the merged file, then there is no subsequent report to look for, so code that as 8888 (Not
* Applicable).

do if DtReportEnd eq DtReportEndFinal.
compute TimeBetweenReports = 8888.
else.
compute TimeBetweenReports = datediff(Next6MoReport,DtReportBeg,"months").
end if.


MISSING VALUES TimeBetweenReports (7777, 8888, 9999).

execute.

* Possible results for TimeBetweenReports: 
* ... blank = a subsequent report was not found for this child
* ... 6 = a subsequent report for this child was found, and it occurred in the next 6-month reporting period 
* ... 12 or more = a subsequent report for this child was found, but it occurred in a later reporting period (i.e., not 
*     the immediate next one). 
* ... 8888 = not applicable (report is in the last reporting period in the merged file).


*Create Age variable, using age at entry if child entered 
* Age at entry (categorical), specified as 0-3 mos, 4-11 mos, 1-5 yrs, 6-10 yrs, 11-16 yrs, 17, 18-21 yrs. 
* Useful for reporting and descriptive statistics. 
********************************

* Calculate age at entry in months.
compute AgeNmos = datediff(DtLatRem,DtBirth,"months").
* Record age in months to desired categories.
recode AgeNmos (216 thru 263=6) (204 thru 216=5) (132 thru 204=4) (72 thru 132=3) (12 thru 72=2) (4 thru 12=1) (0 thru 4=0) (else=sysmis) into AgeNmosyrsCat.
execute.

* Age at entry (interval), specified as 0-3 mos, 4-11 mos, 1 yr ... 21 yrs. 
* Used later in risk adjustment, where each age value is converted to a yes/no (1/0) dummy variable. 
********************************

* Calculate age at entry in years.
compute AgeNyrs = datediff(DtLatRem,DtBirth,"years").
* Children < 1 currently have a value of 0 for AgeNyrs. Need to split these into 0-3 mos (0) mos and 4-11 mos (1). 
* To make room for the 0 and 1 for < 1 children, make a copy of AgeNyrs then shift all subsequent ages, so age1 = 2, age2 = 3, etc. 
compute AgeNmosyrs = AgeNyrs.
* Recode 1 - 21 to 2 - 22. 
recode AgeNyrs (1=2) (2=3) (3=4) (4=5) (5=6) (6=7) (7=8) (8=9) (9=10) (10=11) (11=12) (12=13) (13=14) (14=15) (15=16) (16=17) (17=18) (18=19) (19=20) (20=21) (21=22) INTO AgeNmosyrs.
* If child is 0-3, make AgeNmosyrs = 0.
* If child is 4-11, make AgeNmosyrs = 1.
if AgeNmosyrsCat = 0 AgeNmosyrs = 0.
if AgeNmosyrsCat = 1 AgeNmosyrs = 1.
execute. 

* Calculate age on first day of report period in years.

compute AgeRPTBegyrs = datediff(DtReportBeg,DtBirth,"years").
execute. 

compute ExAge18=0.
if AgeNyrs ge 18 or AgeRPTBegyrs ge 18 ExAge18=1.
exe.

delete variables AgeRPTBegyrs.

* Youth who turn 18 during the 12-month period will not have time in care beyond their
* 18th birthday or moves after their 18th birthday counted. To handle this, for children who 
* turn 18 during the period we replace their discharge date with the date they turned 18. 
* Therefore, the LOS for children who turn 18 during the period will be calculated from date of latest
* removal to date of 18th birthday.
****************************************************

* Calculate child's 18th birthday. 
compute Bday18=DATESUM(DtBirth, 18, "years", 'closest').
variable labels  Bday18 "18th birthday".
variable level Bday18(SCALE).
formats Bday18(ADATE10).
variable width Bday18(10).
execute.

* If 18th birthday occurred during the 12-month period, and child had discharged by the time he turned 18, 
* replace date of discharge with date of 18th birthday.
compute Adjust18=0.
EXECUTE.
do if (Bday18 gt DtReportBeg and Bday18 le DtReportEnd) and (missing(DtDisch) or DtDisch gt Bday18).
compute DtDischAdjusted=Bday18.
compute Adjust18=1.
end if.
execute.
variable labels DtDischAdjusted 'For children who turned 18 during the period and had not discharged by that time, the date of their 18th birthday is their effective date of discharge'.

* Copy date of discharge for remaining children to DtDischAdjusted.
do if sysmis(DtDischAdjusted).
compute DtDischAdjusted=DtDisch.
end if.
execute.
formats DtDischAdjusted (adate10).

* Dropped records.
****************************************************

* It is a dropped record if the 6-month record is missing a DtDisch and a subsequent report was not found for
* this child (TimeBetweenReports = BLANK) or a subsequent report was found for this child but it occurred in 
* a reporting period other than the immediate next one (TimeBetweenReports >= 12 months). 
*Dropped case check should be blank  if child was 18 on first day of the period or at entry or turned 18 during the period.
do if (ExAge18=0)  and (Adjust18=0).
    compute DQ_Dropped=0.
    do if sysmis(DtDisch).
        if sysmis(TimeBetweenReports) OR (TimeBetweenReports ge 12 AND TimeBetweenReports lt 8888) DQ_Dropped = 1.
    end if.
end if.
execute.


* AFCARS IDs don't match from one period to next. 
****************************************************

* It is a non-match if the child was not reported in the next 6-month period (TimeBetweenReports = BLANK) or a
* a subsequent report was found for this child but it occurred in a reporting period other than the immediate next 
* one (TimeBetweenReports >= 12 months). These records are not in error and not a problem unless the 
* percentage of non-matches for the entire state exceeds the DQ limit. 
do if (ExAge18=0)  and (Adjust18=0).
    compute DQ_IDNoMatchNext6Mo=0.
    if sysmis(TimeBetweenReports) OR (TimeBetweenReports ge 12 AND TimeBetweenReports lt 8888) DQ_IDNoMatchNext6Mo = 1.
end if.
execute.

delete variables DtReportBeg1 Next6MoReport TimeBetweenReports.


****************************************************
* PREPARATION FOR DATA QUALITY - AFCARS WITHIN-FILE CHECKS
****************************************************

* These checks are run on each 6-month record in the combined file, within each 6-month report period. 
* #variables are scratch variables - they are created temporarily and then automatically removed. 
 
* Missing date of birth.
* Missing date of latest removal.
* Missing number of placement settings.

compute DQ_missDOB=0.
if sysmis(DtBirth) DQ_missDOB = 1.

*Limit the following DQ checks to children under 18.

do if ExAge18=0.
compute DQ_missDtLatRem=0.
if sysmis(DtLatRem) DQ_missDtLatRem = 1.
compute DQ_missNumPlep=0.
if sysmis(numplep) DQ_missNumPlep = 1.
* Date of birth is after the date of latest removal (produces a negative age of entry).
if not(missing(DtBirth)) and not(missing(DtLatRem)) DQ_DOBgtDtLatRem = 0.
if DtBirth > DtLatRem DQ_DOBgtDtLatRem = 1. 
* Date of birth is after date of discharge (produces a negative age at exit).
if not(missing(DtBirth)) and not(missing(DtDisch)) DQ_DOBgtDtDisch = 0.
if DtBirth > DtDisch DQ_DOBgtDtDisch = 1. 
* Date of latest removal is equal to the date of discharge (produces a LOS of 0 to 24 hrs).
if not(missing(DtDisch)) and not(missing(DtLatRem)) DQ_DtDischeqDtLatRem=0.
if DtDisch = DtLatRem DQ_DtDischeqDtLatRem = 1. 	
* Date of discharge is before the date of latest removal (produces a negative LOS).
if not(missing(DtDisch)) and not(missing(DtLatRem)) DQ_DtDischltDtLatRem=0.
if DtDisch < DtLatRem DQ_DtDischltDtLatRem = 1. 	
* Date of discharge exists but the reason for discharge is missing.
if not(missing(DtDisch)) and not(missing(disreasn))  DQ_missDisreasn=0.
if not(missing(DtDisch)) and missing(disreasn) DQ_missDisreasn = 1.
* Child is on first removal (total # of removals from home to date = 1). 
* Not a problem unless the % of children on their first removal exceeds the DQ limit.
if not(missing(totalrem)) DQ_totalrem1=0. 
if totalrem = 1 DQ_totalrem1 = 1.
end if.
execute.

*Include all children include those over 18 for these.
* Time between date of birth and date of latest removal is > 21 yrs (produces age at entry > 21 yrs).
if not(missing(DtBirth)) and not(missing(DtLatRem)) DQ_gt21DOBtoDtLatRem=0.
compute #AgeN = datediff(DtLatRem,DtBirth,"years").
if #AgeN > 21 DQ_gt21DOBtoDtLatRem = 1. 	
* Time between date of birth and date of discharge is > 21 yrs (produces age at exit > 21 yrs).
if not(missing(DtBirth)) and not(missing(DtDisch)) DQ_gt21DOBtoDtDisch = 0.
compute #AgeX = datediff(DtDisch,DtBirth,"years").
if #AgeX > 21 DQ_gt21DOBtoDtDisch = 1. 	
* Time between date of latest removal and date of discharge is > 21 yrs (produces a LOS > 21 yrs).
* MHMH changed datediff  from days to years.
if not(missing(DtDisch)) and not(missing(DtLatRem)) DQ_gt21DtDischtoDtLatRem = 0.
compute #LOSN = datediff(DtDisch, DtLatRem, "years").
if #LOSN > 21 DQ_gt21DtDischtoDtLatRem = 1. 	
EXECUTE.
*added flag for dtpriordisch after dtlatrem.

compute DQ_dtpriordafterdlm = 0.
if (dtpriordisch gt dtlatrem) DQ_dtpriordafterdlm = 1.
execute.

* Formatting
****************************************************
* As noted earlier, records with a 1 for DQ_IDNoMatchNext6Mo are not in error and not a problem 
* unless the percentage of non-matches for the entire state exceeds the DQ limit. 
value labels DQ_Dropped to DQ_totalrem1
0 'Okay'
1 'Potential problem'.
formats DQ_Dropped to DQ_totalrem1 (F2.0).

variable labels
DQ_Dropped 'Record is missing a date of discharge, suggesting child is still in care, but a record in the next 6-month period does not exist'
DQ_IDNoMatchNext6Mo 'No match for a given recnumbr in the next 6-month period. Not a problem unless the percentage of non-matches for a given state exceeds the DQ limit'
DQ_missDOB 'Date of birth is missing'
DQ_missDtLatRem 'Date of latest removal from home is missing'
DQ_missNumPlep 'Number of placements in current foster care episode is missing'
DQ_DOBgtDtLatRem 'Date of birth is after the date of latest removal from home (produces a negative age of entry)'
DQ_DOBgtDtDisch 'Date of birth is after the date of discharge from most recent foster care episode (produces a negative age at exit)'
DQ_gt21DOBtoDtLatRem 'Time between date of birth and date of latest removal from home is > 21 yrs (produces age at entry > 21 yrs)'
DQ_gt21DOBtoDtDisch 'Time between date of birth and date of discharge from foster care is > 21 yrs (produces age at exit > 21 yrs)'
DQ_gt21DtDischtoDtLatRem 'Time between date of latest removal from home and date of discharge from most recent foster care episode is > 21 yrs (produces a LOS > 21 yrs)'
DQ_DtDischeqDtLatRem 'Date of latest removal from home is the same day as the date of discharge from most recent foster care episode (produces a LOS of 0 days)'
DQ_DtDischltDtLatRem 'Date of discharge from most recent foster care episode is before the date of latest removal from home (produces a negative LOS)'
DQ_missDisreasn 'Date of discharge from most recent foster care episode exists but the reason for discharge is missing'
DQ_totalrem1 'Child is on first removal (total # of removals from home to date = 1). Not a problem unless the percentage of children on their first removal for a given state exceeds the DQ limit' 
DQ_dtpriordafterdlm 'Date of prior discharge is after date of latest removal in same record'.

****************************************************
* CALCULATE DERIVED VARIABLES
****************************************************

* These derived variables are calculated for each record in the file. Some will be used in the indicator-specific 
* syntax; others may be useful for other reporting and analyses.

* Entry and exit years
****************************************************

* FFY child entered.

compute EntryYr=xdate.year(DtLatRem). 
if range(xdate.month(DtLatRem),10,12) EntryYr=EntryYr+1.
* If date of latest removal is missing, recode to 9999.
if DQ_missDtLatRem = 1 EntryYr = 9999.
execute.

* FFY child exited.
compute ExitYr=xdate.year(DtDisch). 
if range(xdate.month(DtDisch),10,12) ExitYr=ExitYr+1.
* If date of discharge is missing, assume child is still in care.
if sysmis(DtDisch) ExitYr = 7777.
execute.

MISSING VALUES EntryYr ExitYr (7777, 8888, 9999).

variable labels 
EntryYr 'FFY child entered'
ExitYr 'FFY child exited care (or 7777 if child is still in care)'.
value labels 
EntryYr 9999 'Cannot calculate (DtLatRem missing)'
/ ExitYr 7777 'Child still in care (DtDisch missing)'.
formats EntryYr ExitYr (F4.0). 


* Handle problem values.
********************************

* If age at entry is negative or > 21 yrs, recode to 8888.
do if DQ_DOBgtDtLatRem = 1 or DQ_gt21DOBtoDtLatRem = 1. 
recode AgeNmos AgeNyrs AgeNmosyrsCat AgeNmosyrs (else = 8888).
end if.
* If age at entry can't be calculated, recode to 9999 (missing DOB or DtLatRem).
do if  DQ_missDOB = 1 or DQ_missDtLatRem = 1.
recode AgeNmos AgeNyrs AgeNmosyrsCat AgeNmosyrs (else = 9999).
end if.

* 9-28-2016 - MH adding missing value assignment to 7777,8888,9999.
MISSING VALUES AgeNmos AgeNyrs AgeNmosyrsCat AgeNmosyrs (7777, 8888, 9999).

execute.
						
* Age at exit (categorical), specified as 0-3, 4-11, 1-5, 6-10, 11-16, 17, 18-21.
* Useful for reporting and descriptive statistics. 
********************************

* Calculate age at exit in months (with 777 for still in care).
compute AgeXmos = datediff(DtDisch,DtBirth,"months").
if sysmis(DtDisch) AgeXmos = 7777. 
* Recode age in months to desired categoies.
recode AgeXmos (7777=7777) (216 thru 263=6) (204 thru 216=5) (132 thru 204=4) (72 thru 132=3) (12 thru 72=2) (4 thru 12=1) (0 thru 4=0) (else=sysmis) into AgeXmosyrsCat.
execute.  					

* 9-28-2016 - MH adding missing value assignment to 7777,8888,9999.
MISSING VALUES AgeXmos (7777, 8888, 9999).

* Age at exit (interval), specified as 0-3 mos, 4-11 mos, 1 yr ... 21 yrs. 
* Used later in risk adjustment, where each age value is converted to a yes/no (1/0) dummy variable. 
********************************

* Calculate age at exit in years (with 777 for still in care)..
compute AgeXyrs = datediff(DtDisch,DtBirth,"years").
if sysmis(DtDisch) AgeXyrs = 7777.  
execute.
* Children < 1 currently have a value of 0. Need to split these into 0-3 mos (0) mos and 4-11 mos (1). 
* To make room for the 0 and 1 for < 1 children, make a copy of AgXNyrs then shift all subsequent ages, so age1 = 2, age2 = 3, etc. 
compute AgeXmosyrs = AgeXyrs.
* Recode 1 - 21 to 2 - 22. 
recode AgeXyrs (1=2) (2=3) (3=4) (4=5) (5=6) (6=7) (7=8) (8=9) (9=10) (10=11) (11=12) (12=13) (13=14) (14=15) (15=16) (16=17) (17=18) (18=19) (19=20) (20=21) (21=22) INTO AgeXmosyrs.
* If child is 0-3, make AgeXmosyrs = 0.
* If child is 4-11, make AgeXmosyrs = 1.
if AgeXmosyrsCat = 0 AgeXmosyrs = 0.
if AgeXmosyrsCat = 1 AgeXmosyrs = 1.
execute.

* Handle problem values.
********************************
* If age at exit can't be calculated, recode to 9999 (missing DOB).
do if DQ_missDOB = 1. 
recode AgeXmos AgeXyrs AgeXmosyrsCat AgeXmosyrs (else = 9999).
end if.

MISSING VALUES AgeXmos AgeXyrs AgeXmosyrsCat AgeXmosyrs  (7777, 8888, 9999).
execute.

* Formatting.
****************************************************

variable labels 
EntryYr 'FFY child entered'
AgeNmos 'Age at entry in months' 
AgeNyrs 'Age at entry in years (Under 1, 1, 2, 3 ... 21)' 
AgeNmosyrs 'Age at entry in months (0-3 mo, 4-11 mo) and years (1, 2, 3 ... 21)' 
AgeNmosyrsCat 'Age at entry in months (0-3 mo, 4-11 mo) and years in categories (1-5, 6-10, etc...)'
AgeXmos 'Age at exit in months'
AgeXyrs 'Age at exit in years (Under 1, 1, 2, 3 ... 21, 7777 for still in care)' 
AgeXmosyrs 'Age at exit in months (0-3 mo, 4-11 mo) and years (1, 2, 3 ... 21, 7777 for still in care)'
AgeXmosyrsCat 'Age at exit in months (0-3 mo, 4-11 mo) and years in categories (1-5, 6-10, ... 7777 for still in care)'.

* Create macro that holds age values.
DEFINE !AgeYrs ( )  
0 '< 1 yr'
1 '1 yr'
2 '2 yrs'
3 '3 yrs'
4 '4 yrs'
5 '5 yrs'
6 '6 yrs'
7 '7 yrs'
8 '8 yrs'
9 '9 yrs'
10 '10 yrs'
11 '11 yrs'
12 '12 yrs'
13 '13 yrs'
14 '14 yrs'
15 '15 yrs'
16 '16 yrs'
17 '17 yrs'
18 '18 yrs'
19 '19 yrs'
20 '20 yrs'
21 '21 yrs'
!ENDDEFINE.

DEFINE !AgeMosYrs ( )  
0 '0-3 mos'
1 '4-11 mos'
2 '1 yr'
3 '2 yrs'
4 '3 yrs'
5 '4 yrs'
6 '5 yrs'
7 '6 yrs'
8 '7 yrs'
9 '8 yrs'
10 '9 yrs'
11 '10 yrs'
12 '11 yrs'
13 '12 yrs'
14 '13 yrs'
15 '14 yrs'
16 '15 yrs'
17 '16 yrs'
18 '17 yrs'
19 '18 yrs'
20 '19 yrs'
21 '20 yrs'
22 '21 yrs'
!ENDDEFINE.

DEFINE !AgeMosYrsCat ( )  
0 '0-3 mos' 
1 '4-11 mos'
2 '1-5 yrs'
3 '6-10 yrs'
4 '11-16 yrs'
5 '17'
6 '18-21 yrs'
!ENDDEFINE.

* Age at entry.
value labels 
AgeNmos
8888 'Invalid (age is negative or > 21)'
9999 'Cannot calculate (DOB or date of latest removal missing)'
/AgeNyrs 
!AgeYrs 
8888 'Invalid (age is negative or > 21)'
9999 'Cannot calculate (DOB or date of latest removal missing)'
/AgeNmosyrs 
!AgeMosYrs 
8888 'Invalid (age is negative or > 21)'
9999 'Cannot calculate (DOB or date of latest removal missing)'
/AgeNmosyrsCat
!AgeMosYrsCat 
8888 'Invalid (age is negative or > 21)'
9999 'Cannot calculate (DOB or date of latest removal missing)'.

* Age at exit.
value labels 
AgeXmos
7777 'Not applicable (child still in care as of the end of the 6-month report)' 
8888 'Invalid (age is negative or > 21)'
9999 'Cannot calculate (DOB or date of latest removal missing)'
/AgeXyrs 
!AgeYrs 
7777 'Not applicable (child still in care as of the end of the 6-month report)' 
8888 'Invalid (age is negative or > 21)'
9999 'Cannot calculate (DOB missing)'
/AgeXmosyrs 
!AgeMosYrs 
7777 'Not applicable (child still in care as of the end of the 6-month report)' 
8888 'Invalid (age is negative or > 21)'
9999 'Cannot calculate (DOB or date of latest removal missing)'
/ AgeXmosyrsCat 
!AgeMosYrsCat 
7777 'Not applicable (child still in care as of the end of the 6-month report)' 
8888 'Invalid (age is negative or > 21)'
9999 'Cannot calculate (DOB missing)'.

formats AgeNmos to AgeXmosyrs (F2.0).

* DisReason1 
****************************************************

* Same as the original reason reported by the state, except when disreasn is missing, uses 7777 for still in care.
* Note: The number of reports where DisReasn1 = 7777 will not equal the number of reports where AgeX = 7777,
* because for DisReasn1 we define still in care as sysmis(disreasn) or disreasn = 0, and for AgeX we define 
* still in care as sysmis(DtDisch). 
compute DisReason1 = disreasn.
if sysmis(disreasn) or disreasn = 0 DisReason1 = 7777.
execute.

MISSING VALUES DisReason1 (7777, 8888, 9999).

variable labels DisReason1 'Discharge reason (or 7777 if child still in care)'.
value labels DisReason1 
1 'Reunification' 
2 'Living with a Relative' 
3 'Adoption' 
4 'Emancipation' 
5 'Guardianship' 
6 'Transfer to another agency' 
7 'Runaway' 
8 'Death of child'
7777 'Child was still in care as of the end of the 6-month report'.
formats DisReason1 (F4.0).

* Create DisReason2
****************************************************
* DisReason2 groups disreasns into four categories: Perm, NonPerm, Death, or still in care.
recode DisReason1 (1,2,3,5=1) (4,6,7=2) (8=3) (7777=7777) into DisReason2.
execute.

MISSING VALUES DisReason2  (7777, 8888, 9999).

variable labels DisReason2 'Discharge reason (perm, non perm, death) or still in care (7777)'.
value labels DisReason2 
1 'Permanency (all four types)' 
2 'Non-Permanency' 
3 'Child died' 
7777 'Child was still in care as of the end of the 6-month report'.
formats DisReason2 (F4.0).

* Total number of removals (categorical)
****************************************************
* not used for CFSR 3 performance, but useful for reporting and descriptive statistics and context data. 

recode totalrem (1=1) (2=2) (3=3) (4 thru Highest=4) (else=sysmis) into TRemCat.
execute.

variable labels TRemCat 'Total number of removals from home (categorical)'.
value labels TRemCat
1 '1' 
2 '2' 
3 '3' 
4 '4 or more'.
formats TRemCat (F2.0).

*Race/ethnicity.

recode 
amiakn asian blkafram hawaiipi white untodetm 
(sysmis=0).
exe.
COUNT
  chrace = amiakn asian blkafram hawaiipi white untodetm  (1)  .
EXECUTE .
do if (chrace=1 and amiakn=1).
compute race=1.
else if (chrace=1 and asian=1).
compute race=2.
else if (chrace=1 and blkafram=1).
compute race=3.
else if (chrace=1 and hawaiipi=1).
compute race=4.
else if (chrace=1 and white=1).
compute race=6.
else if (chrace=1 and untodetm=1).
compute race=7.
else if (chrace>1).
compute race=8.
else if (chrace=0).
compute race=9.
end if.
formats race (f1.0).
value labels race
1'American Indian and Alaskan Native'
2'Asian'
3'Black or African American'
4'Native Hawaiian and Other Pacific Islander'
6'White'
7'Unable to Determine'
8'Two or More'
9'Missing'.
exe.
recode 
hisorgin
(sysmis=0).
do if ((hisorgin=0)and (race=9)).
compute racethnc=9.
else if ((hisorgin=0 and race~=9)).
compute racethnc=race.
else if (hisorgin=1).
compute racethnc=5.
else if ((hisorgin=2) and (race~=9)).
compute racethnc=race.
else if ((hisorgin=2) and (race=9)).
compute racethnc=7.
else if ((hisorgin=3) and (race~=9)).
compute racethnc=race.
else if ((hisorgin=3) and (race=9)).
compute racethnc=7.
end if.
formats racethnc (f1.0).
value labels racethnc
1'American Indian/Alaskan Native-Non Hispanic'
2'Asian-Non Hispanic'
3'Black or African American-Non Hispanic'
4'Native Hawaiian/Other Pacific Islander-Non Hispanic'
5'Hispanic'
6'White-Non Hispanic'
7'Unknown/Unable to Determine'
8'Two or More-Non Hispanic'
9'Missing'.
exe.
recode racethnc (9=sysmis).
exe.

* Save.
****************************************************

save outfile='AFCARS_Comb_FH\CFSR 3 AFCARS source data.sav'
  /COMPRESSED.

* Save output.
output save OUTFILE='Root_FH\5 - SPSS Logs\CFSR 3 - Create AFCARS source data.spv'.



