﻿* Encoding: UTF-8.

    ****************************************************
    ****************************************************
    CFSR 3 - CREATE AFCARS DQ FILES
    ****************************************************
    ****************************************************
    
    * Root folder for the data.
****PLEASE CHANGE THIS FILE PATH BASED ON WHERE YOU UNZIPED AND SAVED THE CFSR 3 FOLDER.****.
    file handle Root_FH /name="C:\CFSR 3\Analysis - DP 2018 Dec".
    
***DO NOT CHANGE FILE HANDLES BELOW****.
    * Folder where CFSR syntax is stored, in particular "createDQFiles.sps" must be in the syntax folder.
    file handle Syntax_FH /name="Root_FH\0 - Syntax".
    *Folder for storing the combine sixmonth file and the combined DQ files.
    file handle AFCARS_Comb_FH  /name="Root_FH\0 - Source Data". 
    *Folder for the outputs of the split procedure related to running the 6month DQ checks.
    *The code will overwrite the entire contents of the Splits folder each time it is run!.
    file handle splits_FH /name="Root_FH\0 - Temp files\Splits". 
    *Folder where the individual DQ results are saved. 
    file handle AFCARS_DQ_FH/name="Root_FH\2 - DQ AFCARS". 
    *Folder where the SPSS logs are saved. 
    file handle SPSSLogs_FH  /name="Root_FH\5 - SPSS Logs". 
    * Folder where fixed files are located (fixed files are generally useful static files that are used in various different sets of code).
    * FixedFiles must have "CFSR 3 AFCARS data quality checks.sav'".
    * FixedFiles must have "States.sav'" (a list of all states by state number, state name, and state abbreviation).
    file handle Fixed_FH /name="Root_FH\1 - Fixed files".

    file handle TableauData_FH /name="Root_FH\6 - Data for Tableau".
    
    *****************************************************************************
    * Step 1: Split source file into separate sav files for each 6-month period (i.e., DtReportEnd).
    *****************************************************************************
    
    * Specifies the location of the Splits folder, which much already exist (referenced later in syntax as "splits"). 
    show handles.
    
    get file 'AFCARS_Comb_FH\CFSR 3 AFCARS source data.sav'.
    dataset name SourceData.

    * Create splitting variable with recognizable name for result files.
    string SixMoPeriod(A10).
    * Last two digits of the year.
    compute SixMoPeriod = char.substr(string(xdate.year(DtReportEnd),F4.0), 3).
    do if xdate.month(DtReportEnd) eq 3.
      compute SixMoPeriod = concat(rtrim(SixMoPeriod), 'A').
    else.
      compute SixMoPeriod = concat(rtrim(SixMoPeriod), 'B').
    end if.
    execute.
    * Break dataset into one file for each 6-mo period and create a list of the files created.
    * Note that anything in the target directory (i.e., Splits) wil be deleted before execution!.
    SPSSINC SPLIT DATASET SPLITVAR=SixMoPeriod
    /OUTPUT DIRECTORY= "splits_FH" DELETECONTENTS=YES
    /OPTIONS NAMES=VALUES nameprefix="source"
    FILELIST="Splits_FH\splits.txt".

*****************************************************************************
* Step 2: Create 6-month DQ files, iterating over each split source file.
*****************************************************************************

* Note: This step calls the separate syntax file, CreateDQFiles.sps., which should be in ...\Syntax.

* The FILELIST parameter below uses the list of files created earlier by SPSSINC SPLIT DATASET.
* If the list of files from that job needs to be changed to include a different set of files, change FILELIST= below 
* to a wildcard syntax INPUTDATA=“directory wildcard specification”, 
* e.g., INPUTDATA="work\DQ AFCARS\splits\*.sav.

* PROCESS FILES iterates over a set of files and applies the SYNTAX file to each one.
* The spv output is also saved in this directory, and a log file that lists progress and any errors is also created 
* or appended to in that directory.

SPSSINC PROCESS FILES  FILELIST="Splits_FH\splits.txt"
SYNTAX="Syntax_FH\createDQFiles.sps" OUTPUTDATADIR="AFCARS_DQ_FH" CONTINUEONERROR=YES
VIEWERFILE= "SPSSLogs_FH\creation.spv" CLOSEDATA=YES
LOGFILE="SPSSLogs_FH\log.txt" MACRONAME="!JOB" LOGFILEMODE=APPEND.

*****************************************************************************
* Step 3: Merge 6-month DQ files and run report.
*****************************************************************************

* Close any existing data files or datasets.
dataset close all. 
new file.

begin program.
import spss, spssaux, os, glob

fh = spssaux.FileHandles()

rdir = fh.resolve('AFCARS_DQ_FH')  # Specifies folder containing .sav files.
fils = sorted(glob.glob(rdir + os.sep + "source_*.sav"))
spss.Submit('''get file "%s".
dataset name DQResults.'''  % (fils.pop(0)))
while True:
    adds = ['/file="%s"' % f for f in fils[:min(len(fils), 49)]]
    spss.Submit("""add files /file=*\n%s""" % "\n".join(adds))
    if len(fils) <=49:
        break
    fils = fils[49:]
end program.

*save data that will be used to eliminate states that fail DQ tests from indicator analyses.
save outfile='AFCARS_DQ_FH\Merged AFCARS DQ files.sav'
  /COMPRESSED.

*save data that will be used for putting DQ results in State profiles.

save translate outfile = 'TableauData_FH\Merged AFCARS DQ.csv'
/FIELDNAMES
/cells=labels
/TYPE=CSV
/keep state stateabb SixMoPeriod DtReportBeg DtReportEnd DtReportEndFinal TotalRecords DQVar ProblemRecords DQResult DQFileXW DQLimit DQCheckShort DQCheckLong PermReEntry PS MALFC DQFailedCheck DQDirection
/replace.


* Save output.
begin program.
import os, re, spss

lastfile = os.path.basename(fils[-1]).split(".")[0]
unit = re.search(r"\d+[AB]", lastfile).group(0)    # identifier for last period - suitable file must exist

spss.Submit(r"""output save OUTFILE='SPSSLogs_FH\CFSR 3 - Create AFCARS DQ files up to %s.spv'.""" % unit)
end program.




