﻿* Encoding: UTF-8.
*********************************************************************************************************************************************************************
NCANDS Data Quality Checks
*********************************************************************************************************************************************************************

* PURPOSE: The purpose of this code is to run the five NCANDS related DQ checks. The 5 DQ checks are as following:
1) Child IDs for victims match across years 
2) Child IDs for victims match across years, but dates of birth and sex do not match
3) Missing age for victims
4) Some victims should have AFCARS IDs in Child File
5) Some victims with AFCARS IDs should match IDs in AFCARS Files

*More information on these checks can be found here: ________________________________________________.

* REQUIREMENTS:
* This code assumes the previous code files have been run. In particular a NCANDS source data file and a AFCARS source data file must exist.
*********************************************************************************************************************************************************************

*********************************************************************************************************************************************************************
Establish File Handles.
*********************************************************************************************************************************************************************.

*The file handle Root_FH represents the root folder for CFSR.
****PLEASE CHANGE THIS FILE PATH BASED ON WHERE YOU UNZIPED AND SAVED THE CFSR 3 FOLDER.****.
file handle Root_FH /name="C:\CFSR 3".


***DO NOT CHANGE FILE HANDLES BELOW****.
*The file handle NCANDS_Comb_FH is an input folder for this code. It should contain the combined NCANDS source files produced by earlier code.
file handle NCANDS_Comb_FH  /name="Root_FH\0 - Source Data". 
*The file handle AFCARS_Comb_FH is an input folder for this code. It should contain the combined AFCARS source files produced by earlier code.
file handle AFCARS_Comb_FH  /name="Root_FH\0 - Source Data". 
* The file handle DQfiles_FH is the output folder for this code. It will contain the results of the DQ checks.
file handle NCANDS_DQ_FH /name= "Root_FH\2 - DQ NCANDS".
*The file handle NCANDS_DQ_XLS_FH .
file handle TableauData_FH /name="Root_FH\6 - Data for Tableau".
*The file handle Fixed_FH is the folder for fixed files (aka static files).
file handle Fixed_FH /name= "Root_FH\1 - Fixed files".
* The file handle SpssLogs_FH is the output folder for the SPSS logs for this code.
file handle SpssLogs_FH /name="Root_FH\5 - SPSS Logs". 
* The file handle Syntax_FH must contain the two "sub-program" code files to run. These are: NCANDS_RecordLevel_DQ.sps and NCANDS_ChildLevel_DQ.sps.
file handle Syntax_FH /name="Root_FH\0 - Syntax".
* The file handle will contain temporary files as a result of running the code. This folder will have its contents erased each time this code is run!.
file handle Splits_FH /name="Root_FH\0 - Temp files\Splits".

*********************************************************************************************************************************************************************
* Load the Source dataset and subset it to those required records and variables.
*********************************************************************************************************************************************************************

* Close any existing data files or datasets.
new file.

GET file='NCANDS_Comb_FH\CFSR 3 NCANDS source data.zsav' 
    /keep= subyr stateabb statetxt rptid chid chage chbdate chsex afcarsid flvictim.
CACHE.

 DATASET NAME DQmainAll.
 DATASET ACTIVATE DQmainAll.

*Select only victims.
select if flvictim = 1.
execute.

*********************************************************************************************************************************************************************.
* Create files needed to run NCANDS DQ checks for all NCANDS children (part 1 of 2). One year of data per file is used for within year data checks.
*********************************************************************************************************************************************************************.

* Break dataset into one file for each year period and create a list of the files created.
* Note that anything in the target directory (i.e., Splits) wil be deleted before execution!.
SPSSINC SPLIT DATASET SPLITVAR=subyr
/OUTPUT DIRECTORY= "Splits_FH" DELETECONTENTS=YES
/OPTIONS NAMES=VALUES nameprefix="DQ_NCANDS_R_" 
FILELIST="Splits_FH\splitsR.txt".

*********************************************************************************************************************************************************************.
* Create files needed to run NCANDS DQ checks for all NCANDS children (part 2 of 2). Two years of data per file are used for cross year data checks.
*********************************************************************************************************************************************************************.
DATASET ACTIVATE DQmainAll2.

*Program that creates a list of unique subyrs found in the datafile and creates a dataset which contains 2 consecutive years for each subyr.
BEGIN PROGRAM.
import spss, spssdata, spssaux, os, glob
s = set()
data = spssdata.Spssdata(indexes=['subyr'])
for case in data:
        s.add(case.subyr)
yearlist = list(s)
yearlist.sort()
del data
for year in yearlist:
    if year == yearlist[-1]:
        spss.Submit("Compute lastyear = 1.")
    else:
        spss.Submit("Compute lastyear = 0.")
    spss.Submit("TEMPORARY.")
    spss.Submit("Select if subyr = " + str(year)+ " OR subyr = " + str(year + 1) + ".")
    spss.Submit("Compute YearToUse = " + str(year) + ".")
    spss.Submit("Save outfile = 'Splits_FH\DQ_NCANDS_C_"+ str(int(year)) + ".sav'.")
END PROGRAM.

*********************************************************************************************************************************************************************.
* Run the NCANDS DQ checks on the above generated  files using SPSSINC process files.
*********************************************************************************************************************************************************************.

* Run DQ checks that are appropriate for record level/within year checks.
SPSSINC PROCESS FILES  FILELIST="Splits_FH\splitsR.txt"
SYNTAX="Syntax_FH\NCANDS_RecordLevel_DQ.sps" OUTPUTDATADIR="NCANDS_DQ_FH" CONTINUEONERROR=YES
VIEWERFILE= "SpssLogs_FH\NCANDS_DQ_R_creation.spv" CLOSEDATA=YES
LOGFILE="SpssLogs_FH\NCANDS_DQ_R_log.txt" MACRONAME="!JOB" LOGFILEMODE=APPEND.

* Run DQ checks that are appropriate for child level /between year checks.
SPSSINC PROCESS FILES  INPUTDATA="Splits_FH\DQ_NCANDS_C_*.sav"
SYNTAX="Syntax_FH\NCANDS_ChildLevel_DQ.sps" OUTPUTDATADIR="NCANDS_DQ_FH" CONTINUEONERROR=YES
VIEWERFILE= "SpssLogs_FH\NCANDS_DQ_C_creation.spv" CLOSEDATA=YES
LOGFILE="SpssLogs_FH\NCANDS_DQ_C_log.txt" MACRONAME="!JOB" LOGFILEMODE=APPEND.

*********************************************************************************************************************************************************************.
* Combine the state/year level results of the DQ checks into a single file.
*********************************************************************************************************************************************************************.

begin program.
import spss, spssaux, os, glob

fh = spssaux.FileHandles()

rdir = fh.resolve('NCANDS_DQ_FH')  # Specifies folder containing .sav files.
fils = sorted(glob.glob(rdir + os.sep + "DQ_NCANDS_*.sav"))
spss.Submit('''get file "%s".
dataset name DQResults.'''  % (fils.pop(0)))
while True:
    adds = ['/file="%s"' % f for f in fils[:min(len(fils), 49)]]
    spss.Submit("""add files /file=*\n%s""" % "\n".join(adds))
    if len(fils) <=49:
        break
    fils = fils[49:]
end program.
 EXECUTE.

*********************************************************************************************************************************************************************.
* Merge the results with the fixed file NCANDS data quality checks.sav which contains threshold and other variables required to calculate DQ test pass/fail.
*********************************************************************************************************************************************************************.

Get file='Fixed_FH\CFSR 3 NCANDS data quality checks.sav'.
sort cases by DQVAR.
Save outfile='Fixed_FH\CFSR 3 NCANDS data quality checks.sav'.

DATASET ACTIVATE DQResults.
SORT CASES by DQVAR.
 EXECUTE.

match files 
    /FILE=DQResults  
    /TABLE='Fixed_FH\CFSR 3 NCANDS data quality checks.sav' 
    /BY DQVar. 
execute.
Dataset name DQRandI.
Dataset close DQResults.

COMPUTE DQFailedCheck=0.
DO IF DQDirection = "<".
    if DQResult < DQLimit DQFailedCheck=1.
ELSE IF DQDirection = ">".
    if DQResult > DQLimit DQFailedCheck=1.
ELSE if Upper(DQDirection) = "N".
    if DQResult = 0 DQFailedCheck=1.
END IF.

STRING Period (A12).
IF DQFileXW = "Within file" Period = STRING(Subyr,F4.0).
IF (DQFileXW = "Cross file" AND DQVAR = "AFCARSMatch") Period = STRING(Subyr,F4.0).
IF (DQFileXW = "Cross file" AND DQVAR ~= "AFCARSMatch") Period = CONCAT(STRING(Subyr,F4.0), "-", STRING(Subyr+1,F4.0)).

save outfile='NCANDS_DQ_FH\Merged NCANDS DQ files.sav'
  /COMPRESSED
  /keep statetxt stateabb subyr DQVAR DQSum DQN DQResult DQFailedCheck Period DQFileXW DQDirection DQLimit DQCheckShort DQCheckLong MALFC MALREC.

String MatchField (A8).
Compute MatchField = "".

save translate outfile = 'TableauData_FH\Merged NCANDS DQ.csv'
/FIELDNAMES
/rename = (statetxt = state)
/TYPE=CSV
/keep state DQVAR stateabb DQResult DQFailedCheck Period DQFileXW DQDirection DQLimit DQCheckShort DQCheckLong MALFC MALREC MatchField
/replace.


GET FILE ='NCANDS_DQ_FH\Merged NCANDS DQ files.sav'.

DATASET CLOSE DQRandI.

*********************************************************************************************************************************************************************.
* Combine the individual child/year level results of the DQ checks into a single file.
*********************************************************************************************************************************************************************.

begin program.
import spss, spssaux, os, glob

fh = spssaux.FileHandles()

rdir = fh.resolve('NCANDS_DQ_FH')  # Specifies folder containing .sav files.
fils = sorted(glob.glob(rdir + os.sep + "IND_DQ_NCANDS_*.sav"))
spss.Submit('''get file "%s".
dataset name INDResults.'''  % (fils.pop(0)))
while True:
    adds = ['/file="%s"' % f for f in fils[:min(len(fils), 49)]]
    spss.Submit("""add files /file=*\n%s""" % "\n".join(adds))
    if len(fils) <=49:
        break
    fils = fils[49:]
end program.

DATASET ACTIVATE INDResults.

save outfile='NCANDS_DQ_FH\Merged NCANDS IND files.sav'
  /COMPRESSED.

NEW FILE.
DATASET CLOSE INDResults.


