﻿* Encoding: UTF-8.
****************************************************
CFSR 3: OBSERVED PERFORMANCE
- RECURRENCE OF MALTREATMENT.
****************************************************.
     
set printback=on.

*The file handle Root_FH represents the root folder for the CFSR data.
file handle Root_FH /name="C:\CFSR 3\".
*The file handle CombinedFiles_FH is the input folder for this code. It should contain the combined NCANDS and combined AFCARS source files produced by earlier code.
file handle NCANDS_Comb_FH  /name="Root_FH\0 - Source Data". 
* The file handle DQfiles_FH is the output folder for the results of the DQ checks.
file handle NCANDS_DQ_FH /name= "Root_FH\2 - DQ NCANDS".
* The file handle SpssLogs_FH  is the output folder for the SPSS logs.
file handle SpssLogs_FH /name="Root_FH\5 - SPSS Logs". 
* The file handle Observed_Child_FH is the output folder for the primary results of this code (results on a per child basis).
file handle Observed_Child_FH /name="Root_FH\3 - Performance observed child".
* The file handle Observed_State_FH is the output folder for the results of this code (aggregated to a state-level basis).
file handle Observed_State_FH /name="Root_FH\3 - Performance observed state".


********************************************************************************************************
SPECIFY THE 12-MONTH COHORT WHOSE OUTCOME WILL BE ASSESSED
********************************************************************************************************
     ********  START USER INPUT ********.
define YYYY () XXXX   
!enddefine.
define YYstr () "XX"   
!enddefine.
define YY2str () "XX"   
!enddefine.

     ********  END USER INPUT ********

********************************************************************************************************
GET SOURCE FILES
********************************************************************************************************

GET file='NCANDS_Comb_FH\CFSR 3 NCANDS source data.zsav' 
    /keep= subyr stateabb state statetxt rptid rptdt rptcnty inciddt chid chage maldeath chbdate chsex afcarsid flvictim.
CACHE.
DATASET NAME Indicator.

****************************************************
* FILE PREP
****************************************************

* Verify the records represent the FYs of interest.
select if subyr eq YYYY OR subyr eq YYYY+1.
execute.

* Select only records with report dates on or after the beginning of the first fiscal year.
select if rptdt ge date.mdy (10,01,YYYY-1).
execute.

 * Select down to only records with a victim flag = 1. (Only use victimized children).
Select if flvictim = 1.
execute.

* Create variable that indicates the user-specified 12-month cohort.
string TwelveMoCohort (A6).
compute TwelveMoCohort = concat("FY",String(YYYY,F4)).
execute.

*select children only ago 0 thru 17.
select if chage ge 0 AND chage le 17.
EXECUTE.

********************************************************************************************************
* State Level DQ - FLAG AND DROP STATES THAT EXCEED DQ LIMITS ON DQ CHECKS
********************************************************************************************************

get file 'NCANDS_DQ_FH\Merged NCANDS DQ files.sav'.
dataset name DQResultsN window=front.

* Prevent accidentally overwriting file.
dataset copy  DQIndicatorN.
dataset activate DQIndicatorN.
dataset close DQResultsN.

*Create a flag variable for records applicable for the current run.
if ((malREC eq 1) and (DQFileXW eq 'Cross file') and (subyr = YYYY)) DQchecksApplyN =1. 
if ((malREC eq 1) and (DQFileXW eq 'Within file') and (subyr ge YYYY) AND (subyr le YYYY+1)) DQchecksApplyN =1. 
select if DQchecksApplyN =1.
sort cases by stateabb subyr.
execute.

*Create a new table with state abbreviation and a flag variable for each state denoting the number of relevant DQ checks that were failed.
DATASET DECLARE DQ_State_Exclusion.
aggregate
  /OUTFILE=DQ_State_Exclusion
  /BREAK=stateabb
  /DQFailedCheckN_sum=SUM(DQFailedCheck).

*  Merge the number of DQ checks failed into the original indicator data file.
dataset activate indicator.
sort cases by stateabb.
match files 
    /FILE=*
    /TABLE='DQ_State_Exclusion' 
    /BY stateabb.
execute.

*Close datasets that are no longer needed and exclude any state that failed at least one DQ check.
dataset close DQIndicatorN.
dataset close DQ_State_Exclusion.

select if DQFailedCheckN_sum eq 0.
execute.
delete variables DQFailedCheckN_sum.

****************************************************
* Individual Level DQ - FLAG AND DROP STATES THAT EXCEED DQ LIMITS ON DQ CHECKS
****************************************************


get file 'NCANDS_DQ_FH\Merged NCANDS IND files.sav'.
dataset name DQResults_Ind window=front.

SELECT IF subyr = YYYY.
execute.

sort cases by stateabb chid.

Dataset activate indicator.

sort cases by stateabb chid rptdt.

match files 
    /FILE=*
    /TABLE='DQResults_Ind' 
    /BY stateabb chid.
execute.

Dataset close DQResults_Ind.

SELECT IF sysmis(notsamechild) or notsamechild=0.
EXECUTE.

****************************************************
* DERIVED VARIABLES AND PROCESSING
****************************************************

* Determine the time between reports for 14 day rollup.
do if (stateabb = lag(stateabb) and chid = lag (chid)).
compute timebetweenrpts = datediff ( rptdt,lag(rptdt), 'days').
end if.

* Apply 14 day roll up.
recode timebetweenrpts (sysmis = 9999).
select if timebetweenrpts gt 14.

* Count the relative report position for a victim.
compute posit = 1.
if (stateabb eq lag(stateabb) and chid eq lag(chid))posit = lag(posit)+1.
execute.

* Exclude fatalities if they happened in the first report. * Possible issue with maltreatment death based of Federal Register explanation.
compute keep = 1.
if (posit eq 1 and maldeath eq 1) keep = 0.
execute.
select if keep = 1.
execute.
 
*Exclude records with matching incident dates.
compute sameincd =0.
do if (stateabb = lag(stateabb) and chid = lag (chid)).
if (not(missing(inciddt)) AND not(missing(lag(inciddt))) AND (inciddt eq lag(inciddt))) sameincd =1.
end if.
execute.
select if sameincd ne 1.
execute.


compute posit2 = 1.
if (stateabb eq lag(stateabb) and chid eq lag(chid))posit2 = lag(posit2)+1.
execute.
* Select only the first two reports for a victim. Recurrence is only computed using the first two reports, so 
* subsequent reports are of no interest. 
select if (posit2 le 2).
execute.

compute missingincd = 0.
if missing(inciddt) missingincd = 1.
execute.

* Aggregate to create a unique file.
dataset declare SourceData_Aggregated.
aggregate outfile = 'SourceData_Aggregated'
  /break = stateabb state chid
  /TwelveMoCohort = first(TwelveMoCohort)
  /FY = first(subyr)
  /county = first(rptcnty)
  /dob = first (chbdate)
  /ageinyears = first (chage)
  /initialrpt = first(rptdt)
  /lastrpt = last(rptdt)
  /initialincdt = first(inciddt)
  /lastincdt = last(inciddt)
  /missingincddt = max(missingincd)
  /statetxt = first(statetxt).
dataset activate SourceData_Aggregated.
dataset close Indicator.

 if (missingincddt eq 1) initialincdt = initialrpt.
 if (missingincddt eq 1) lastincdt = lastrpt.
 execute.

****************************************************
* Flag cases where recurrence occurs.
****************************************************

* Compute the time between the first and the last report.
compute mthsbetweenrpts = datediff(lastrpt,initialrpt, 'months')..
if (initialrpt eq lastrpt) mthsbetweenrpts = $sysmis.

* If the time betweeen reports is greater than or equal to 0 (we have already applied 14 day roll up rule and removed reports) and less than 12 months, recurrence occurs.
compute recur = 0.
if (initialrpt <> lastrpt) AND (mthsbetweenrpts ge 0 and mthsbetweenrpts lt 12) recur = 1.
execute.

* If the initial incident date is the same as the last incident date, set recur to 0.
do if (initialincdt ge 1 and lastincdt ge 1).
if (initialincdt eq lastincdt)recur = 0.
end if.
execute.

variable labels recur 'Maltx recurrence numerator - Child had a recurrence of maltx within 12 months of his initial report'.
value labels recur 
   0 'No recurrence occurred' 
   1 'Yes recurrence occurred'.
formats recur (F1.0).

* Select only records where the first report date occurred in the first fiscal year (this uses two submission periods).
select if (initialrpt le date.mdy(09,30,YYYY)).
execute.

*rename variables (chid ageinmonths ageinyears recur = ChildID AgeMos AgeYrs Num_Child).
rename variables (chid recur = ChildID Num_Child).

****************************************************
* CALCULATE AGE AT INITIAL REPORT
****************************************************

* Compute age in months at the time of the initial report.
compute ageinmonths = datediff(initialrpt,dob, 'months').

* Some children have a negative ageinmonths due to dob > initialrpt. These are plausible and involve allegations 
* concerning children in utero. Recode to 0 for 0-3 months age group, which includes unborn. 
recode ageinmonths (lowest thru -1 = 0).
variable labels ageinmonths 'Age in months at initial report'.
value labels ageinmonths 0 'less than 1 month includes unborn'.
variable labels ageinyears 'Age in years at initial report'.
value labels ageinyears 0 'Less than1 year includes unborn' 99 'Unknown age'.

* Recode age in months to desired categories.
* Age at initial victimization (interval), specified as 0-3 mos, 4-11 mos, 1 yr ... 21 yrs.
* Used later in risk adjustment, where each age value is converted to a yes/no (1/0) dummy variable. 
recode ageinmonths (216 thru 263=6) (204 thru 216=5) (132 thru 204=4) (72 thru 132=3) (12 thru 72=2) (4 thru 12=1) (0 thru 4=0) (else=sysmis) into AgeMosyrsCat.

* Children < 1 currently have a value of 0 for AgeYrs. Need to split these into 0-3 mos (0) mos and 4-11 mos (1). 
* To make room for the 0 and 1 for < 1 children, make a copy of AgeYrs then shift all subsequent ages, so 
* age1 = 2, age2 = 3, etc. 
compute AgeMosyrs = ageinyears.
* Recode 1 - 21 to 2 - 22. 
if (ageinyears ge 1) AgeMosyrs = ageinyears + 1.
if AgeMosyrsCat = 0 AgeMosyrs = 0.
if AgeMosyrsCat = 1 AgeMosyrs = 1.
execute.

variable labels 
ageinmonths 'Age at initial report in months' 
ageinyears 'Age at initial report in years (Under 1, 1, 2, 3 ... 21)' 
AgeMosyrs 'Age at initial report in months (0-3 mo, 4-11 mo) and years (1, 2, 3 ... 21)' 
AgeMosyrsCat 'Age at initial report in months (0-3 mo, 4-11 mo) and years in categories (1-5, 6-10, etc...)'.

* Create macro that holds age values.
DEFINE !AgeYrs ( )  
0 '< 1 yr'
1 '1 yr'
2 '2 yrs'
3 '3 yrs'
4 '4 yrs'
5 '5 yrs'
6 '6 yrs'
7 '7 yrs'
8 '8 yrs'
9 '9 yrs'
10 '10 yrs'
11 '11 yrs'
12 '12 yrs'
13 '13 yrs'
14 '14 yrs'
15 '15 yrs'
16 '16 yrs'
17 '17 yrs'
18 '18 yrs'
19 '19 yrs'
20 '20 yrs'
21 '21 yrs'
!ENDDEFINE.

DEFINE !AgeMosYrs ( )  
0 '0-3 mos'
1 '4-11 mos'
2 '1 yr'
3 '2 yrs'
4 '3 yrs'
5 '4 yrs'
6 '5 yrs'
7 '6 yrs'
8 '7 yrs'
9 '8 yrs'
10 '9 yrs'
11 '10 yrs'
12 '11 yrs'
13 '12 yrs'
14 '13 yrs'
15 '14 yrs'
16 '15 yrs'
17 '16 yrs'
18 '17 yrs'
19 '18 yrs'
20 '19 yrs'
21 '20 yrs'
22 '21 yrs'
!ENDDEFINE.

DEFINE !AgeMosYrsCat ( )  
0 '0-3 mos' 
1 '4-11 mos'
2 '1-5 yrs'
3 '6-10 yrs'
4 '11-16 yrs'
5 '17'
6 '18-21 yrs'
!ENDDEFINE.

value labels 
ageinmonths
8888 'Invalid (age is negative or > 21)'
9999 'Cannot calculate (DOB missing)'
/ageinyears
!AgeYrs 
8888 'Invalid (age is negative or > 21)'
9999 'Cannot calculate (DOB missing)'
/AgeMosyrs 
!AgeMosYrs 
8888 'Invalid (age is negative or > 21)'
9999 'Cannot calculate (DOB missing)'
/AgeMosyrsCat
!AgeMosYrsCat 
8888 'Invalid (age is negative or > 21)'
9999 'Cannot calculate (DOB missing)'.

formats ageinyears ageinmonths AgeMosyrsCat AgeMosyrs (F3.0).

****************************************************
* Create additional variables needed for Stata or Tableau.
****************************************************

String Period (A10).
Compute Period = concate("FY",YYstr,"-FY",YY2str).
Execute.

* State-level data.
sort cases by state (A) ChildID (A).
aggregate
  /OUTFILE=* MODE=ADDVARIABLES
  /PRESORTED
  /BREAK=state
  /Num_State=SUM(Num_Child) 
  /Den_State=N.
compute Perf_State = Num_State / Den_State.
compute Perf_State_MP = (Num_State / Den_State) * 100.
execute.

* National-level data.
aggregate
  /OUTFILE=* MODE=ADDVARIABLES
  /BREAK=
  /Num_Nation=SUM(Num_Child)
  /Den_Nation=N.
compute Perf_Nation = Num_Nation / Den_Nation.
compute Perf_Nation_MP =  (Num_Nation / Den_Nation) * 100.
execute.
formats Perf_State Perf_Nation (F8.5).

variable labels 
Den_State 'Maltx recurrence denominator - Number of children in state who were victimized in the first 12-month period'
Num_State 'Maltx recurrence numerator - Among children in the denominator, number of children in state who had a recurerence of maltx within 12 months of their initial victimization'
Perf_State 'Maltx recurrence - Percentage of children in state who had a recurrence of maltx within 12 months of their initial victimization'
Den_Nation 'Maltx recurrence denominator - Number of children in nation who were victimized in the first 12-month period'
Num_Nation 'Maltx recurrence numerator - Among children in the denominator, number of children in nation who had a recurerence of maltx within 12 months of their initial victimization'
Perf_Nation 'Maltx recurrence - Percentage of children in nation who had a recurrence of maltx within 12 months of their initial victimization'.

****************************************************
* OUTPUT FILES
****************************************************

* Save.
save outfile='Observed_Child_FH\CFSR 3 - Observed perf for maltx recurrence '  + YYstr + YY2str + '.sav'
   /compressed.

* Save file for STATA (for multi-level modeling).
save translate OUTFILE='Observed_Child_FH\CFSR 3 - Observed perf for maltx recurrence ' + YYstr + YY2str + '.dta'
  /TYPE=STATA
  /VERSION=13
  /EDITION=SE
  /MAP
  /REPLACE
  /RENAME (AgeMosyrs = ChildAge)
  /KEEP=state stateabb statetxt TwelveMoCohort ChildID Num_Child ChildAge Den_State 
Num_State Perf_State Den_Nation Num_Nation Perf_Nation.

* Create file with one record per state holding observed performance for the 12-month cohort.
compute numstates = 1.
if (state eq lag(state))numstates = 0.
execute.

dataset copy OneRecordPerState.
dataset activate OneRecordPerState.
select if (numstates=1).
execute.
dataset activate OneRecordPerState.

string Indicator (A30).
execute.
compute Indicator = "Maltx recurrence".
execute.

save outfile='Observed_State_FH\CFSR 3 - Observed perf for maltx recurrence State file ' + YYstr + YY2str + '.sav'
   /keep state stateabb statetxt Indicator TwelveMoCohort Perf_State.

dataset activate SourceData_Aggregated.
dataset close OneRecordPerState.

output save OUTFILE='SpssLogs_FH\CFSR 3 - Observed perf for maltx recurrence ' + YYstr + YY2str + '.spv'.


****************************************************
ALL DONE.
****************************************************


