﻿* Encoding: UTF-8.

****************************************************
CFSR 3 - CALCULATE OBSERVED PERFORMANCE 
- PLACEMENT STABILITY
****************************************************

*The file handle Root_FH represents the root folder for the CFSR data.
***** CHANGE THE ROOT FILE HANDLE TO MATCH THE WHERE YOU HAVE PLACED THE CFSR 3 FOLDER ON YOUR COMPUTER***.
file handle Root_FH /name="C:\CFSR 3".

file handle AFCARS_Comb_FH  /name="Root_FH\0 - Source Data". 

file handle AFCARS_DQ_FH /name= "Root_FH\2 - DQ AFCARS".

file handle SpssLogs_FH /name="Root_FH\5 - SPSS Logs". 

file handle Observed_Child_FH /name="Root_FH\3 - Performance observed child".

file handle Observed_State_FH /name="Root_FH\3 - Performance observed state".

* Open source data.

GET file='AFCARS_Comb_FH\CFSR 3 AFCARS source data.sav'.
dataset name SourceData window=front.
cache.
* Prevent accidentally overwriting source data.
dataset copy  Indicator.
dataset activate Indicator.
dataset close SourceData.

* Delete unused variables. 

delete variables amiakn asian blkafram hawaiipi white untodetm hisorgin clindis mr vishear phydis dsmIII othermed everadpt ageadopt manrem phyabuse sexabuse neglect aaparent
daparent aachild dachild childis chbehprb prtsdied prtsjail nocope abandmnt relinqsh housing placeout casegoal ctkfamst ctk1yr ctk2yr fosfamst fcctk1yr fcctk2yr rf1amakn
rf1asian rf1blkaa rf1nhopi rf1white rf1utod hofcctk1 rf2amakn rf2asian rf2blkaa rf2nhopi rf2white rf2utod hofcctk2 ivefc iveaa ivaafdc ivdchsup xixmedcd ssiother noa fcmntpay DtReview 
DtLatRemTrans DtTPRMom DtTPRDad DtDischTrans.

****************************************************
SPECIFY THE 12-MONTH COHORT WHOSE OUTCOME WILL BE ASSESSED
****************************************************

* Identify the 12-month cohort of interest (e.g., children entering in 13B14A)
* by entering "AB" or "BA" for PeriodType and the year the 12-month period ends (for YYYY and YYstr). 

* AB period (A file + B file) spans Oct 1 - Sept 30 of the following year.
* BA period (B file + A file) spans Apr 1 - Mar 31 of the following year.

* Examples:
* ( * ) represents a FFY
* Children who entered     PeriodType  YYYY   YYstr
* 09B10A                        BA              2010     10
* 10A10B *                      AB              2010     10
* 10B11A                        BA              2011     11
* 11A11B *                      AB              2011     11
* 11B12A                        BA              2012     12  
* 12A12B *                      AB              2012     12 
* 12B13A                        BA              2013     13         
* 13A13B *                      AB              2013     13
* 13B14A                        BA              2014     14
* etc.

********  START USER INPUT ********

define PeriodType () "XX"
!enddefine.
define YYYY () XXXX.   
!enddefine.
define YYstr () "XX" 
!enddefine. 

********  END USER INPUT ********


****************************************************
CREATE DATE PARAMETERS
****************************************************

* Start date of a 12-month period (10/1/YYYY or 4/1/YYYY).
* End date of a 12-month period (3/31/YYYY or 9/30/YYYY).

* Placement stability involves following children who entered during the 12-month period specified earlier.
* Identify the start date (DtPeriodBeg) and end date (DtPeriodEnd) that defines this 12-month period. 

do if PeriodType = "AB".
compute DtPeriodBeg=date.mdy(10,01,YYYY-1).
compute DtPeriodEnd=date.mdy(09,30,YYYY).
compute DtEndFirst6mo=date.mdy(03,31,YYYY).
end if.
do if PeriodType = "BA".
compute DtPeriodBeg=date.mdy(04,01,YYYY-1).
compute DtPeriodEnd=date.mdy(03,31,YYYY).
compute DtEndFirst6mo=date.mdy(09,30,YYYY-1).
end if.
execute.
formats DtPeriodBeg DtPeriodEnd (adate10).

variable labels 
DtPeriodBeg 'Start date of the first 12-month period specified'
DtPeriodEnd 'End date of the first 12-month period specified'.

if (DtReportBeg ge DtPeriodBeg) and (DtReportEnd le DtPeriodEnd) ReportedDuringPeriod = 1.
execute.


* Select the records to keep.
select if ReportedDuringPeriod=1.
execute.

delete variables ReportedDuringPeriod.


* Create variable that indicates the user-specified 12-month cohort (e.g., "BA14" represents "13B14A").
string TwelveMoCohort (A4).
compute TwelveMoCohort = concat(PeriodType,YYstr).
execute.

****************************************************
REMOVE STATES THAT EXCEED DQ LIMITS
****************************************************

* The AFCARS cross-file checks (Dropped records and AFCARS IDs don't match from one period to the next)
* apply only to the first 6-month period (DtReportBeg = DtPeriodBeg). The AFCARS within-file checks apply to 
* both periods (DtReportBeg ge DtPeriodBeg AND DtReportBeg le DtPeriodEnd). 

* Open the data quality results done previously.
get file= 'AFCARS_DQ_FH\Merged AFCARS DQ files.sav'.
dataset name DQResults window=front.

* Prevent accidentally overwriting CFSR 3 data quality results file.
dataset copy  DQIndicator.
dataset activate DQIndicator.
dataset close DQResults.

****************************************************
CREATE DATE PARAMETERS (In The DQ File)
****************************************************

do if PeriodType = "AB".
compute DtPeriodBeg=date.mdy(10,01,YYYY-1).
compute DtPeriodEnd=date.mdy(09,30,YYYY).
compute DtEndFirst6mo=date.mdy(03,31,YYYY).
end if.
do if PeriodType = "BA".
compute DtPeriodBeg=date.mdy(04,01,YYYY-1).
compute DtPeriodEnd=date.mdy(03,31,YYYY).
compute DtEndFirst6mo=date.mdy(09,30,YYYY-1).
end if.
execute.
formats DtPeriodBeg DtPeriodEnd (adate10).

variable labels 
DtPeriodBeg 'Start date of the first 12-month period specified'
DtPeriodEnd 'End date of the first 12-month period specified'.

* Select the AFCARS checks for this indicator and DQ results for the 6-month period(s).
* ... Cross file checks (apply only to the first 6-month period).
if PS = 1 and DQFileXW = "Cross file" AND (DtReportBeg eq DtPeriodBeg) DQChecksApply = 1.
* ... Within file checks (apply to both 6-month periods).
if PS = 1 and DQFileXW = "Within file" AND (DtReportBeg ge DtPeriodBeg) and (DtReportEnd le DtPeriodEnd) DQChecksApply = 1.
execute.

* For each state, for the checks that apply, sum the number of checks it failed.
sort cases by state.
select if DQChecksApply = 1.
execute.
dataset declare DQstate.
aggregate
  /OUTFILE=DQState
  /BREAK=state
  /DQFailedCheck_sum=SUM(DQFailedCheck).

execute.

* ... Merge in all variables from DQIndicator, then keep only DQFailedCheck_sum. 
dataset activate Indicator.
sort cases by state.
match files 
    /FILE=*
    /TABLE='DQState' 
    /BY state
    /KEEP state to TwelveMoCohort DQFailedCheck_sum.
execute.

dataset close DQIndicator.
dataset close DQstate.

* Select only states that met the DQ checks.
select if DQFailedCheck_sum = 0.
execute.

delete variables DQFailedCheck_sum.

* Count the number of states remaining.
compute numstates = 1.
if (state eq lag(state))numstates = 0.
execute.

****************************************************
SELECT APPLICABLE RECORDS 
****************************************************


* Select only the most recent 6-month record reported for each child, for each episode.
****************************************************

* This will create an episode-level file. Episodes are distinguished by the date of latest removal from home.  
* A child has a record in each 6-month submission until he discharged, dropped, or was still in care as of 
* the last reporting period we have in the file. We only want one record per child, per episode, and the record we pick 
* is the last one reported for that episode. This record will contain the most recent data available that describes 
* the child's episode (e.g., placement moves, age at entry, etc.). 

* Flag the records to keep.
sort cases BY ChildID (A) DtLatRem (A).
match files
  /FILE=*
  /BY ChildID DtLatRem
  /LAST=Flag_LastRpt4Ep.
variable labels Flag_LastRpt4Ep 'last 6-month report we received for this episode (based on DtLatRem)'.
value labels Flag_LastRpt4Ep 0 'Duplicate Case' 1 'Primary Case'.
variable level Flag_LastRpt4Ep (ORDINAL).
execute.

* Select the last 6-month record for each child, for each episode. 
select if Flag_LastRpt4Ep=1.
execute.

delete variables Flag_LastRpt4Ep.

* Select only children who entered during the 12-month period specified earlier. 
****************************************************

* This is the cohort of children for whom performance will be examined.

* Flag the records to keep.
if ((DtLatRem ge DtPeriodBeg) and (DtLatRem le DtPeriodEnd)) Entered = 1.
recode Entered (sysmis = 0).
execute.

variable labels Entered 'Child entered care during the specified 12-month period'.
value labels Entered 
0 'No'
1 'Yes'. 

* Select the Entered records to keep. 
select if Entered = 1.
execute.

* Handling multiple episodes in the 12-month period. 
****************************************************

* For placement stability, all of a child's episodes during the 12-month period are considered. 
* For example, if a child has two entries in the 12-month period, data from both episodes are used.

****************************************************
* REMOVE RECORDS WITH DQ PROBLEMS RELEVANT TO THIS MEASURE.
****************************************************
 
* Do not delete dropped records if they occur in the last period needed to calculate observed performance. 
* This ensures we use only data from submissions required to observe the cohort. 
if DtReportEnd eq DtPeriodEnd DQ_Dropped = 0.
execute.

* Flag records with a problem (i.e., = 1) for any of the DQ checks used for Placement Stability, 
* except DQ_IDNoMatchNext6Mo and DQ_totalrem1. 
compute DQ_Indicator=0.
if any(1,DQ_DOBgtDtDisch, DQ_DOBgtDtLatRem, DQ_Dropped, DQ_DtDischeqDtLatRem, DQ_DtDischltDtLatRem, DQ_gt21DOBtoDtDisch, DQ_gt21DOBtoDtLatRem, DQ_gt21DtDischtoDtLatRem, DQ_missDOB, DQ_missDtLatRem, DQ_missNumPlep, DQ_dtpriordafterdlm) 
DQ_Indicator=1.  
execute.

* Delete records with a DQ problem(s).
select if DQ_Indicator=0.
execute.

* Delete the DQ variables as they are no longer needed. 
delete variables DQ_Dropped to DQ_totalrem1 DQ_Indicator. 


****************************************************
CALCULATE LENGTH OF STAY FOR EACH EPISODE
****************************************************

* Identify children who exited during the 12-month period (needed later).
****************************************************
* Identify children who have more than one episode during the 12-month period.
****************************************************

sort cases by childid dtreportbeg.
compute duplicate_child eq 0.
execute.
if (childid eq lag(childid)) duplicate_child=1.
execute.

*label each episode.
compute position=1.
if (duplicate_child=1) position = lag(position)+1. 
execute.

*flag all records for children with more than one episode.
aggregate 
outfile=* mode=ADDVARIABLES
/break state childid
/dupchild = max(duplicate_child).

*pull dtpriordisch from into prior records.

PRESERVE.
set workspace 200000.
create TempDtPriorDisch=Lead(DtPriorDisch,1).
execute.
RESTORE.

*pull number of total removals reported in subsequent episode into record from prior episode.
PRESERVE.
set workspace 200000.
create nexttotrem=Lead(totalrem,1).
execute.
RESTORE.

*if child has more than one episode, calculate the different between total removals reported in second episode and total removals reported in first episode.

if (dupchild=1 and position=1) changerem = (nexttotrem - totalrem).
execute.

*flag cases/records where we need to look at dtpriordisch.
compute flag_useprior = 0.
if not(missing(tempdtpriordisch)) AND missing(dtdisch) AND dupchild eq 1 AND position eq 1 flag_useprior = 1. 
execute. 

*flag when dtpriordisch falls in the period of analysis.
if flag_useprior eq 1 and tempdtpriordisch ge dtperiodbeg and tempdtpriordisch le dtperiodend priorinperiod eq 1. 
execute. 

*flag if dtpriordisch falls in first 6-month period of the analysis.
if priorinperiod eq 1 and tempdtpriordisch le dtendfirst6mo priorinfirst eq 1. 
execute.

*flag when it is acceptable to use dtpriordisch to compute length of stay.
if flag_useprior eq 1 and tempdtpriordisch ge dtperiodbeg and tempdtpriordisch le DtEndFirst6Mo AND changerem eq 1 usepriordisch eq 1.
execute.

*drop episodes where to number of total removals changed by more than 1 between the first and second epsiode reported for the child. 
compute drop_changerem =0.
if dupchild eq 1 and position eq 1 and changerem ne 1 and missing(dtdisch) drop_changerem = 1.
execute.

select if drop_changerem ne 1.
execute.

*set DtDischNextRecord equal to tempdtpriordisch is child/episode meets all requirements to use date prior discharge for computing LOS.
if  usepriordisch eq 1 AND adjust18 eq 1 and changerem eq 1 and tempdtpriordisch gt dtlatrem AND (TempDtPriorDisch lt DtDischAdjusted) AND (DtDischAdjusted eq Bday18) DtDischNextRecord = TempDtPriorDisch.
if usepriordisch eq 1 and missing(DtDischAdjusted) AND changerem eq 1 and tempdtpriordisch gt dtlatrem DtDischNextRecord=TempDtPriorDisch.
execute.

*We can only use dtdischnextrecord to compute los if the date falls in the first six month period.
* If the date falls in the second six month period we have to use the last day of the first 6-month period to compute LOS.
if (DtDischNextRecord ge dtperiodbeg and DtDischNextRecord le DtEndFirst6mo) DtDischAdjusted=DtDischNextRecord.
execute.


if ((DtDischAdjusted ge DtPeriodBeg) and (DtDischAdjusted le DtPeriodEnd)) Exited = 1.
recode Exited (sysmis = 0).
execute.
variable labels Exited 'Child exited care during the specified 12-month period'.
value labels Exited 
0 'No'
1 'Yes'. 
* LOS (days) between date of latest removal from home and date of discharge (or last date in the 12-month period).
****************************************************

compute LOSLatRemDays = $sysmis.
* For children who exited during the 12-month period, LOS is time between DtLatRem and DtDischAdjusted.
do if Exited=1. 
compute LOSLatRemDays=datediff(DtDischAdjusted,DtLatRem,"days").
end if.
execute.

* For children who did not exit during the 12-month period, LOS is time between DtLatRem and DtPeriodEnd.
do if Exited=0. 
compute LOSLatRemDays=datediff(DtPeriodEnd,DtLatRem,"days").
end if.
execute.
*For children who had more than one episode during the 12-month period and the discharge date of the first episode fell in the second 6-month period,
*LOS is the time between DtLatRem and the last day of the first 6-month period.

do if Exited=0 AND dupchild=1 AND position=1 and (DtEndFirst6mo gt DtLatRem). 
compute LOSLatRemDays=datediff(DtEndFirst6mo,DtLatRem,"days").
end if.
execute.

*For children who had more than one episode during the 12-month period, the LOS for the second episode if the child did not discharge from that episode
*is the time between the removal date for that episode and the last day of the 12-month period of analysis.
do if Exited=0 AND dupchild=1 AND position=2 and (DtPeriodEnd gt DtLatRem). 
compute LOSLatRemDays=datediff(DtPeriodEnd,DtLatRem,"days").
end if.
execute.


variable labels 
LOSLatRemDays 'LOS - Days between DtLatRem and DtDischAdjusted (for discharged) or last date in the 12-month period (for still in care)'.

formats LOSLatRemDays (F4.0).

****************************************************
IDENTIFY AND REMOVE EPISODES THAT MEET EXCLUSION CRITERIA
****************************************************

* The denominator for this indicator is the sum of children's LOS (days) across *all* episodes during the 
* 12-month period. So if a child entered twice during the 12-month periods, his total LOS is the LOS from his
* first episode plus the LOS from his second episode. However, when summing a child's LOS over all his episodes
* in the 12-month period, we want to exclude:
* 1) episodes in which the child entered at age 18 or more and
* 2) *complete* episodes with LOS < 8 days. If the episode has a LOS < 8 days, but the child is still in care, we 
* want to keep this. These short but ongoing episodes represent entries that occurred near the end of the 
* 12-month period and continued past it (i.e., child was still in care).

* Complete episodes with LOS < 8 days *** THIS WILL CORRECTLY ELIMINATE STAYS FOR CHILDREN WHO ENTERED WITHIN 8 DAYS OF 18th BDAY.  
compute ExLOS8 = 0.
if (Exited = 1 and Adjust18 ne 1 and LOSLatRemDays < 8) ExLOS8 = 1.
execute.

variable labels 
ExLOS8 'Complete episode has a LOS < 8 days'.

value labels ExLOS8 
0 'No'
1 'Yes'.
execute.
formats ExLOS8 (F1.0).


* Select the records to keep.
select if (ExLOS8 = 0 and ExAge18 = 0).
execute.

****************************************************
ADJUST NUMBER OF PLACEMENTS
****************************************************

* We don't want to count the first placement (which reflects the child's entry into care).

compute numplepAdjust = numplep -1.
if (DtCurSet gt DtDischAdjusted and numplepAdjust ge 1) numplepadjust = numplepadjust - 1.
execute.

*formatting.
variable labels numplepAdjust 'Number of placement settings, excluding the first placement setting associated with the childs entry'.

formats numplepAdjust (F2.0).

*flag records for same child with conflicting information.
sort cases by ChildID dtreportbeg.
if (dtlatrem ne lag(dtlatrem) AND childID eq lag(ChildID) and missing(lag(dtdischadjusted))) flagCase eq 1.
execute.

*flag if child with two different dates of latest removal doesn't have a dtpriordisch or the date is from before the period began.
if flagcase eq 1 AND (missing(dtpriordisch) or (dtpriordisch lt dtperiodbeg)) flagcase2 eq 1.
execute.

**set flag to remove earliest record in the period for the child.

aggregate
  /OUTFILE=* MODE=ADDVARIABLES
  /BREAK=ChildID
  /Flagrptremove2 = max(flagcase2).

compute dropreport eq 0.
if (flagrptremove2 eq 1 AND position eq 1) dropreport eq 1.
execute.

select if dropreport eq 0.
execute.

*flag if child has two records that show overlapping episodes and remove earliest report in period for child.
if (dtlatrem ne lag(dtlatrem)) AND childID eq lag(ChildID) AND missing(dtpriordisch) AND dtlatrem lt lag(dtdischadjusted) flagpriorrec = 1.
execute.

aggregate
  /OUTFILE=* MODE=ADDVARIABLES
  /BREAK=ChildID
  /removerec = max(flagpriorrec).

compute removerecord = 0.
if (position eq 1 AND removerec eq 1) removerecord = 1.
execute.

select if removerecord eq 0.
execute.



***********************
SUM EACH CHILD's LOS and NUMBER OF PLACEMENTS ACROSS EPISODES
****************************************************

sort cases BY ChildID.
aggregate
  /OUTFILE=* MODE=ADDVARIABLES
  /BREAK=ChildID
  /Den_Child=SUM(LOSLatRemDays) 
  /Num_Child=SUM(numplepAdjust).

variable labels 
Den_Child 'Placement stability denominator - Childs total length of stay (days) across all episodes in the 12-month period'
Num_Child 'Placement stability numerator - Childs total number of placements (numplepAdjust) across all episodes in the 12-month period'.

formats Den_Child (F3.0) Num_Child (F2.0).



* Some children may have a total LOS = 0 days. 
****************************************************

* These are children who entered on the last day of the 12-month period. At this point, they have not been in care 
* for a full 24 hours. In addition, placement rates for these children cannot be calculated when their LOS 
* (i.e., denominator) is 0.

select if Den_Child <> 0.
execute.

* Some children may have more moves than days in care, and therefore a placement rate > 1.  
****************************************************

* These are likely DQ issues. In the national file, affects only ~6 children. In addition, placement rates for these 
* children cannot be used as rate could be > 1.

compute MovesgtDays = 0.
if numplepAdjust > Den_Child MovesgtDays = 1.
execute.

select if MovesgtDays = 0.
execute.

* Number of placement settings (categorical)
****************************************************
* Not used for CFSR 3 performance, but may be useful for reporting and descriptive statistics. 

recode Num_Child (1=1) (2=2) (3=3) (4=4) (5=5) (6=6) (7=7) (8=8) (9=9) (10 thru Highest=10) (else=sysmis) into Num_ChildCat.
execute.

variable labels Num_ChildCat 'Total number of placement settings (categorical)'.
value labels Num_ChildCat
1 '1' 
2 '2' 
3 '3' 
4 '4' 
5 '5' 
6 '6' 
7 '7' 
8 '8' 
9 '9' 
10 '10 or more'.
formats Num_ChildCat (F2.0). 

***************************************************
AGGREGATE FILE TO ONE RECORD PER CHILD
****************************************************

* For children with more than one episode, retain the age at entry and totalrem associated with 
* his earliest episode (based on DtLatRem) in the 12-month period. 
****************************************************

* Only age at entry is used in risk adjustment, but totalrem may be of interest for reporting and 
* descriptive statistics.
sort cases by ChildID(A) DtLatRem(A).
execute.
aggregate
  /OUTFILE=* MODE=ADDVARIABLES OVERWRITEVARS=YES
  /PRESORTED
  /BREAK=ChildID
  /TRemCat_first=FIRST(TRemCat) 
  /AgeNmosyrsCat_first=FIRST(AgeNmosyrsCat) 
  /AgeNmosyrs_first=FIRST(AgeNmosyrs). 

* Select one record per child. 
****************************************************
* Select the most recently reported one for the 12-month period, although the decision is arbitrary since 
* the data used to measure performance is identical in both records (i.e., Num_Child, Den_Child, Age...first).

* Flag the records to keep.
sort cases BY ChildID (A) DtReportEnd (A).
match files
  /FILE=*
  /BY ChildID 
  /LAST=Flag_LastRpt4Ch.
variable labels Flag_LastRpt4Ch 'last 6-month report we received for this child (based on DtReportEnd)'.
value labels Flag_LastRpt4Ch 0 'Duplicate Case' 1 'Primary Case'.
variable level Flag_LastRpt4Ch (ORDINAL).
execute.

* Select the records to keep. 
select if Flag_LastRpt4Ch=1.
execute.

delete variables Flag_LastRpt4Ch.

****************************************************
* SUMMARY STATISTICS
****************************************************

* Create variables holding state and national performance.
****************************************************

* Child-level data.
compute Perf_Child = (Num_Child / Den_Child).
compute Perf_Child_MP = (Num_Child / Den_Child) * 1000.
execute.

* State-level data.
sort cases by state (A) ChildID (A).
aggregate
  /OUTFILE=* MODE=ADDVARIABLES
  /PRESORTED
  /BREAK=state
  /Num_State=SUM(Num_Child) 
  /Den_State=SUM(Den_Child)
  /N_State=N.
compute Perf_State = (Num_State / Den_State).
compute Perf_State_MP = (Num_State / Den_State) * 1000.
execute.

* National-level data.
aggregate
  /OUTFILE=* MODE=ADDVARIABLES
  /BREAK=
  /Num_Nation=SUM(Num_Child)
  /Den_Nation=SUM(Den_Child) 
  /N_Nation=N.
compute Perf_Nation = (Num_Nation / Den_Nation).
compute Perf_Nation_MP = (Num_Nation / Den_Nation) * 1000.
execute.

formats Perf_Child Perf_State Perf_Nation (F6.5).

variable labels 
N_State 'Number of children in the states file'
Den_State 'Placement stability denominator - Total number days children were in care in state across all episides in the 12-month period'
Num_State 'Placement stability numerator - Total number of placement moves in state children experienced while in care across all episodes in the 12-month period'
Perf_State_MP 'Placement stability - Placement rate per 1,000 days in care (state)'
N_Nation 'Number of children in the national file'
Den_Nation 'Placement stability denominator - Total number days children were in care in nation across all episides in the 12-month period'
Num_Nation 'Placement stability numerator - Total number of placement moves in nation children experienced while in care across all episodes in the 12-month period'
Perf_Nation_MP 'Placement stability - Placement rate per 1,000 days in care (nation)'.


****************************************************
* OUTPUT FILES
****************************************************

* Save.
save outfile='Observed_Child_FH\CFSR 3 - Observed perf for placement stability ' + PeriodType + YYstr + '.sav'
   /compressed.

* Save file for STATA (for multi-level modeling).
rename variables (AgeNmosyrs_first = ChildAge). 
save translate OUTFILE='Observed_Child_FH\CFSR 3 - Observed perf for placement stability ' + PeriodType + YYstr + '.dta'
  /TYPE=STATA
  /VERSION=13
  /EDITION=SE
  /MAP
  /REPLACE
  /KEEP=state stateabb statetxt TwelveMoCohort DtPeriodBeg DtPeriodEnd ChildID Num_Child Den_Child ChildAge N_State Den_State 
Num_State Perf_State Perf_State_MP N_Nation Den_Nation Num_Nation Perf_Nation Perf_Nation_MP.
rename variables (ChildAge = AgeNmosyrs_first). 

* Save file with one record per state holding observed performance for the 12-month cohort.
compute numstates = 1.
if (state eq lag(state))numstates = 0.
execute.

dataset copy OneRecordPerState.
dataset activate OneRecordPerState.
select if (numstates=1).
execute.
dataset activate OneRecordPerState.

string Indicator (A30).
execute.
compute Indicator = "Placement stability".
execute.

save outfile='Observed_State_FH\CFSR 3 - Observed perf for placement stability State file ' + PeriodType + YYstr + '.sav'
   /keep state stateabb statetxt Indicator TwelveMoCohort DtPeriodBeg DtPeriodEnd Perf_State.

dataset activate Indicator.
dataset close OneRecordPerState.

output save OUTFILE='SpssLogs_FH\CFSR 3 - Observed perf for placement stability ' + PeriodType + YYstr + '.spv'.

***********************************************************
ALL DONE.
**************************************************************




