* VERSION HISTORY
****************************

****************************
****************************
* CFSR 3: RISK-STANDARDIZED PERFORMANCE  
* - MALTREATMENT IN FOSTER CARE
****************************
****************************

* This syntax is provided for informational purposes only. It requires access to 
* child-level data from all states, the District of Columbia, and Puerto Rico.  

* The following syntax is used to calculate an individual state's performance 
* for a recent cohort of children against the national standard and the 
* historical cohorts (from all other states) that were used to establish the 
* national standard. Although this syntax will calculate RSPs and data for all 
* states, the only results of interest are for the state being evaluated 
* (MyState "XX"). 

****************************
* SPECIFY THE STATE AND 12-MONTH COHORT WHOSE PERFORMANCE IS BEING ASSESSED
****************************

* Note: For the 12-month cohort specified (CurrentCohort), there must exist a 
* file for that same cohort in "C:\cfsr3\Performance observed child". This file 
* will contain observed performance for a recent cohort of children (from all 
* states). For example, if the user specifies CurrentCohort = AB14, there must 
* be a file called "CFSR 3 - Observed perf for maltx in care AB14.dta."

set more off
set trace off
*set min_memory 16G
****************************
* USER INPUT REQUIRED
****************************

* Specify root folder where your files and folders are. 
* This folder must have these subfolders:
* ... \Performance modeled
* ... \Performance observed child
* ... \Fixed files

local dirstub "C:\CFSR 3\Analysis - DP 2019 July"


* Specify 12-month cohort
local CurrentCohortList AB17 
*local CurrentCohortList AB12 AB13 AB14 AB15 AB16

* Specify state(s) to run. 
* Option 1 is to list them below in the local line (e.g.):
local statelist CO MT
local statetxtlist Colorado Montana
*      local statelist MA NH VT ME CT RI PA NY 
* Option 2 is to list them in a dataset (which can be conveniently used for the 
* other indicators).
*use "`dirstub'/1 - Fixed files/States 2017.dta", clear
*local statelist
*local statetxtlist
*forvalues i = 1/`=_N' {
*     local statelist `"`statelist' `"`=stateabb[`i']'"'"'
*	 local statetxtlist `"`statetxtlist' `"`=statetxt[`i']'"'"'
*}
****************************
* RUN SYNTAX FROM HERE TO END
****************************
foreach CurrentCohort of local CurrentCohortList {
*local numelements = wordcount("`statelist'")
local numelements : word count `statelist'

*forvalue i = 1/`numelements'{
forvalue i = 1/1{
 local MyState : word `i' of `statelist'
 local MyStateTxt : word `i' of `statetxtlist'
* create log file. 
log using "`dirstub'/4 - Performance modeled/RSP for maltx in care `MyState' `CurrentCohort'.txt", text replace


****************************
* GET SOURCE FILE and PREP FILE
****************************

* Open the current cohort file. This file contains observed performance for a 
* recent cohort of children (from all states). 
use "`dirstub'/3 - Performance observed child/CFSR 3 - Observed perf for maltx in care `CurrentCohort'.dta", clear
keep if stateabb == `"`MyState'"' 


* Flag if state requested is missing from the current cohort data file (usually due to failing DQ).
quietly count
local DQState = 0
if r(N) == 0 {
	local DQState = 1
}

tempfile holding
save `holding'

* Open the historical cohort file. This file contains the fixed national 
* standard for the indicator (Perf_Nation) and the observed performance for the 
* 12-month historical cohort of children that was used to establish the national 
* standard. We will be assessing the state's performance with its most recent 
* cohort (currently stored in 'holding') against the national standard and 
* historical cohorts from all other states. This syntax assumes the historical
* cohort file is saved in the Fixes files folder of the dirstub path defined earlier.
macro list
use "`dirstub'/1 - Fixed files/CFSR 3 - NS Observed perf for maltx in care AB13.dta", clear

quietly summarize ChildAge, detail
local ChildAge_Med = r(p50)
local Perf_NationNS = Perf_Nation[1]
local Perf_NationNS_MP = Perf_Nation_MP[1]

* IF State requested does not have a current cohort score, creat a special DQ 
* output for that state rather than run the rest of the code.
if `DQState' == 1 {
	clear
	set obs 10	
	generate stateabb = `"`MyState'"'
	generate state = `"`MyStateTxt'"'
	generate Indicator_new = "Maltreatment in care (victimizations/100,000 days in care)"
	generate NS = string(round(`Perf_NationNS_MP',0.01))
	generate TwelveMoCohort = "FY" + "20" + substr(`"`CurrentCohort'"',1,2)
	generate Period = substr(`"`CurrentCohort'"',3,2) + "AB,FY" + substr(`"`CurrentCohort'"',3,2)
	generate DataUsed = substr(`"`CurrentCohort'"',3,2) + "AB,FY" + substr(`"`CurrentCohort'"',3,2)
	generate Format = "Number"
	generate RSP_NS = ""
	generate RSP_Range = ""
	generate RSP_Range_Formatted = ""
	generate RSP_Pip = ""
	generate ValueType = "DataUsed2" in 1
	replace ValueType = "Den_State" in 2
	replace ValueType = "Num_State" in 3
	replace ValueType = "Perf_NationRnd" in 4
	replace ValueType = "Perf_StateRnd" in 5
	replace ValueType = "RSP_NS2" in 6
	replace ValueType = "RspLowRnd" in 7
	replace ValueType = "RSP_Range2" in 8
	replace ValueType = "RSPRnd" in 9
	replace ValueType = "RSPUppRnd" in 10
	generate Result = 0
	replace Result = round(`Perf_NationNS',0.001) in 4
	generate Result_String = "DQ"
	replace Result_String = DataUsed in 1
	replace Result_String = "" in 4
	replace Result_String = "" in 6
	generate MatchField = ""
	*Save file
	save "`dirstub'/4 - Performance modeled/CFSR 3 - RSP for maltx in care - `CurrentCohort' `MyState'.dta", replace
	outsheet stateabb state	Indicator_new NS TwelveMoCohort Period DataUsed Format RSP_NS RSP_Range RSP_Range_Formatted RSP_Pip ValueType Result Result_String MatchField ///
	using "`dirstub'/6 - Data for Tableau/Stata Output/RSP_MaltxInCare_Summary `CurrentCohort' `MyState'.csv", comma nolabel replace
	export excel stateabb state	Indicator_new NS TwelveMoCohort Period DataUsed Format RSP_NS RSP_Range RSP_Range_Formatted RSP_Pip ValueType Result Result_String MatchField ///
	using "`dirstub'/6 - Data for Tableau/Stata Output/RSP_MaltxInCare_Summary `CurrentCohort' `MyState'.xlsx", nolabel replace firstrow (variables)
	*close log and return to the top of the loop
	 log close
	 continue
}

drop if stateabb == `"`MyState'"'
append using `holding'
* For the state being evaluated, replace the value it has for national observed
* performance (which is based on the current cohort for all states) with the 
* fixed NS for this indicator. The level of precision matters for the 
* calculations RSP, RSPUpp, and RSPLow matters.

* IMPORTANT - For Rate indicators (PS & MALTX FC), replace Perf_Nation_MP as 
* well. For the Rate indicators, the _MP version is used as the NS when 
* comparing to the RSP. 
replace Perf_Nation = `Perf_NationNS' if stateabb == `"`MyState'"'
replace Perf_Nation_MP = `Perf_NationNS_MP' if stateabb == `"`MyState'"'
sort state

* Describe dataset
* Pick one observation per state
* Count and verify number of states in file
* Calculate desired direction for performance on this indicator
* Verify desired 12-month cohort has been selected

describe
egen pickone = tag(stateabb)
count if pickone
gen Passing = "Below NS"
tab Perf_Nation_MP
tab Perf_Nation
tab TwelveMoCohort

****************************
* PREDICT OUTCOME
****************************

* Run multi-level poisson model predicting victimizations (Num_Child) given 
* one or more foster care episodes with a particular duration in days 
* (Den_Child). Adjust for child age at entry or on first day (ChildAge).

********  POSSIBLE USER INPUT NEEDED ********

* The xtmepoisson command requires the user to enter the value of X in  
* ib(X).ChildAge, where X represents the reference group for children in the 
* national dataset. The median age is used as the reference group. The ib 
* function creates a dummy variable for each age value.  
* To get the median age value run: 

summarize ChildAge, detail


* The coded value to use is the one next to "50%." Enter that value in ib(X) 
* below. 

timer on 1
xtmepoisson Num_Child ib(`ChildAge_Med').ChildAge, baselevels exposure(Den_Child) || state:, variance
timer off 1
timer list 1
timer clear

predict xb, xb 
predict re, reffects 
predict rese, reses 

* xb = natural logarithm of child's predicted number of victimizations based on 
* child's age and the number of days of the child's foster care episode(s) 
* (i.e., exposure to the event of interest), but without considering the child's 
* home state (i.e., ignoring the state's random effect)

* re = state's random effect (shift in the natural logarithm of the child's
* predicted number of victimizations after considering the child's home state; 
* aka, Empirical Bayes intercept)

* rese - standard error of state's random effect

****************************
* CALCULATE RISK-STANDARDIZED PERFORMANCE
****************************

* 1. Calculate PREDICTED number of victimizations in each state
****************************

* The predicted number of victimizations based on the state's performance with 
* its observed case mix. This is our best prediction of future performance, 
* assuming no change in case mix or policy. It is calculated as the sum of each 
* child's predicted number of victimizations, which is a complex function of 
* the child's value of xb (based on child's age and episode duration in days) 
* and his or her state's specific random effect).

sort state
gen double Child_Pred = exp(xb+re)
by state: egen double Num_Pred = total(Child_Pred) 

* 2. Calculate EXPECTED number of victimizations in each state
****************************

* The expected number of victimizations based on the nation's performance with 
* the state's case mix. This represents how many victimizations we would expect
* for the state's children if they were treated in an "average" state. It is 
* calculated as the sum of each child's predicted number of victimizations (xb), 
* including the *average* intercept of all states (the average of the 
* random effects across states is zero). 

gen double Child_Exp = exp(xb)
by state: egen double Num_Exp = total(Child_Exp)

* 3. Calculate risk-standardized ratio and risk-standardized performance
****************************

* Calculate ratio of predicted to expected
* Multiply ratio by national observed performance (i.e., the national standard)
* to get RSP
gen double Ratio_PE = Num_Pred / Num_Exp
gen double RSP = (Num_Pred / Num_Exp) * Perf_Nation
gen double RSP_MP = RSP * 100000

* 4. Calculate 95% confidence intervals for RSP
****************************

* Upper CI of the RSP
sort state
gen double UppNum = exp(xb+re+(1.96*rese))
by state: egen double UppNumSum = total(UppNum)
gen double UppDen = exp(xb)
by state: egen double UppDenSum = total(UppDen)
gen double RSPUpp = (UppNumSum/UppDenSum) * Perf_Nation
gen double RSPUpp_MP = RSPUpp * 100000

* Low CI of the RSP
gen double LowNum = exp(xb+re-(1.96*rese))
by state: egen double LowNumSum = total(LowNum)
gen double LowDen = exp(xb)
by state: egen double LowDenSum = total(LowDen)
gen double RSPLow = (LowNumSum/LowDenSum) * Perf_Nation
gen double RSPLow_MP = RSPLow * 100000

****************************
* COMPARE STATE'S RSP TO NATIONAL STANDARD (NS)
****************************

* When comparing the state's RSP (actually, the CIs) to the national observed
* performance, use rounded versions. For rate indicators (maltreatment in 
* foster care and placement stablility), use Perf_Nation_MP, RSRLow_MP, etc. 
* instead of the unmultiplied rate (e.g., Perf_Nation, RSRLow, which have too 
* many trailing 0s such that the rounded rates will all = 0; e.g., .00000631 
* will get rounded to .00; whereas 6.3094187 will get rounded to 6.31).

* Round the national observed performance 
clonevar Perf_NationRnd = Perf_Nation_MP
replace Perf_NationRnd = round(Perf_NationRnd,0.01)

* Round the CIs of the RSP
clonevar RSPLowRnd = RSPLow_MP
clonevar RSPUppRnd = RSPUpp_MP
replace RSPLowRnd = round(RSPLowRnd,0.01)
replace RSPUppRnd = round(RSPUppRnd,0.01)

* Compare CI's of the RSP relative to national observed performance
gen RSP_NS = "No diff"
replace RSP_NS = "Met" if RSPUppRnd < Perf_NationRnd
replace RSP_NS = "Not met" if RSPLowRnd > Perf_NationRnd

* Count number of states meeting, not meeting, and no different from NS
egen RSP_Met=total(RSP_NS=="Met" & pickone)
egen RSP_NotMet=total(RSP_NS=="Not met" & pickone)
egen RSP_NotDif=total(RSP_NS=="No diff" & pickone)

* Count number of states that must engage in a PIP
gen RSP_Pip = ""
replace RSP_Pip = "No PIP" if RSP_NS == "Met"
replace RSP_Pip = "No PIP" if RSP_NS == "No diff"
replace RSP_Pip = "PIP" if RSP_NS == "Not met"


* Create variable holding median age to include in output. The coded age values 
* are one higher than the actual age (e.g., a coded value of 7 represents an 
* actual age of 6), so the median age = coded value for median age * minus 1.  
sort state
by state: egen MedAge = median(ChildAge)
replace MedAge = MedAge -1

* Save
save "`dirstub'/4 - Performance modeled/CFSR 3 - RSP for maltx in care - `CurrentCohort' `MyState'.dta", replace

****************************
* REPORTING 
****************************

* Export to excel the state's results, formatted for profile

gen Indicator_new = "Maltreatment in care (victimizations/100,000 days in care)"

* Round the RSP and Perf_State to 2 decimals (so it's, e.g., 3.57; avoids having 
* to format it later in Excel to show only 2 decimals)
gen RSPRnd = round(RSP_MP,0.01) 
gen Perf_StateRnd = round(Perf_State_MP,0.01)

* gen RSP_new = round(RSP,0.001)*100 (e.g., "40.1")
gen RSP_Range = string(RSPLowRnd) + "-" + string(RSPUppRnd)
gen RSP_Range_Formatted = string(RSPLowRnd) + "-" + string(RSPUppRnd)
gen Format = "Number"
if substr(`"`CurrentCohort'"',1,2) ==  "AB"{
               gen Period = substr(`"`CurrentCohort'"',3,2) + "AB,FY" + substr(`"`CurrentCohort'"',3,2)
}
else if substr(`"`CurrentCohort'"',1,2) == "BA" {
               gen Period = string(real(substr(`"`CurrentCohort'"',3,2))-1) + "B" + substr(`"`CurrentCohort'"',3,2) + "A"
}

if substr(`"`CurrentCohort'"',1,2) ==  "AB"{
               gen DataUsed = substr(`"`CurrentCohort'"',3,2) + "A-" + substr(`"`CurrentCohort'"',3,2) + "B, FY" + substr(`"`CurrentCohort'"',3,2)
}
else if substr(`"`CurrentCohort'"',1,2) == "BA" {
               gen DataUsed = string(real(substr(`"`CurrentCohort'"',3,2))-1) + "B-" + string(real(substr(`"`CurrentCohort'"',3,2))+2) + "A"
}
*These variables only exist to ensure that there is both a column and record with this data after reshaping below, and because tableau.
gen NS = string(round(`Perf_NationNS_MP',0.01))
gen RSP_Range2 = RSP_Range_Formatted
gen RSP_NS2 = RSP_NS
gen DataUsed2 = DataUsed
gen MatchField = ""

* RSP (e.g., .0035744702...)
* RSP_MP (e.g., 3.5744702)
* gen RSP_new = round(RSP_MP,0.01) (e.g., "3.57")
* gen RSPRange = string(RSPLowRnd) + " - " + string(RSPUppRnd)
* Perf_State_MP (e.g., 3.5348582)

* outsheet stateabb Indicator_new TwelveMoCohort RSP_new RSPRange Perf_NationRnd RSP_NS RSP_Pip ///
* if (stateabb == `"`MyState'"' & pickone==1) using "`dirstub'/Performance modeled/Profile_RSP_MaltxInCare_Summary `CurrentCohort' `MyState'.csv", comma nolabel replace

outsheet stateabb statetxt Indicator_new NS TwelveMoCohort Period DataUsed DataUsed2 Format RSP_NS RSP_NS2 RSP_Range RSP_Range_Formatted RSP_Range2 RSP_Pip Den_State Num_State Perf_StateRnd RSPLowRnd RSPRnd RSPUppRnd Perf_NationRnd MatchField ///
if (stateabb == `"`MyState'"' & pickone==1) using "`dirstub'/4 - Performance modeled/RSP_MaltxInCare_Summary `CurrentCohort' `MyState'.csv", comma nolabel replace


import delimited "`dirstub'/4 - Performance modeled/RSP_MaltxInCare_Summary `CurrentCohort' `MyState'.csv", clear case(preserve)

rename (Den_State Num_State Perf_StateRnd RSPLowRnd RSPRnd RSPUppRnd Perf_NationRnd) Result=
rename (RSP_Range2 RSP_NS2 DataUsed2) str=

reshape long Result str, i(stateabb statetxt) j(ValueType) string
*reshape long result str, i(stateabb indicator_new ns twelvemocohort period data_used format rsp_ns rsp_range rsp_range_formatted rsp_pip) j(ValueType) string
replace Result = 0 if missing(Result)
 

* Use a string version of the State's name rather than a labeled numeric version.
rename (str) Result_String
*rename (statetxt) state


outsheet stateabb	state	Indicator_new	NS	TwelveMoCohort	Period	DataUsed	Format	RSP_NS	RSP_Range	RSP_Range_Formatted	RSP_Pip	ValueType	Result	Result_String MatchField ///
 using "`dirstub'/6 - Data for Tableau/Stata Output/RSP_MaltxInCare_Summary `CurrentCohort' `MyState'.csv", comma nolabel replace
 
export excel stateabb state	Indicator_new NS TwelveMoCohort Period DataUsed Format RSP_NS RSP_Range RSP_Range_Formatted RSP_Pip ValueType Result Result_String MatchField ///
 using "`dirstub'/6 - Data for Tableau/Stata Output/RSP_MaltxInCare_Summary `CurrentCohort' `MyState'.xlsx", nolabel replace firstrow (variables)
	
*close log and return to the top of the loop
 log close
}
}
