﻿* Encoding: UTF-8.

file handle output /name="Output".

* Open source data.  JOB_INPUTFILE is a file handle defined for each iteration
   by PROCESS FILES.
get file 'JOB_INPUTFILE'.
dataset name DQmain.

define !out() !quote(!concat(!unquote(!eval(!JOB_OUTPUTDATADIR)),"\",
    !unquote(!eval(!JOB_DATAFILEROOT)),
    ".sav")) 
!enddefine.


DATASET ACTIVATE DQmain.

DATASET DECLARE DQ_PerChild. 
AGGREGATE 
  /OUTFILE='DQ_PerChild' 
  /BREAK=stateabb (A) chid (A) subyr (D)
  /statetxt = first(statetxt)
  /chsex_min=MIN(chsex) 
  /chsex_max=MAX(chsex) 
  /chbdate_min=MIN(chbdate) 
  /chbdate_max=MAX(chbdate)
  /chage_min=MIN(chage) 
  /chage_max=MAX(chage)
  /YearToUse=min(YearToUse)
 /lastyear=max(lastyear).

********************************************************************************************
* DQ Check: Child IDs for victims match across years 
********************************************************************************************.
dataset activate DQ_PerChild.

Compute sameid = 0.
DO IF (subyr+1) = lag(subyr) AND chid = lag(chid).
    Compute sameid = 1.
END IF.
EXECUTE.

if lastyear eq 1 sameid eq $sysmis.
execute.

*Aggregate.
DATASET DECLARE DQ_IDM. 
AGGREGATE 
  /OUTFILE='DQ_IDM' 
  /BREAK=stateabb(A) subyr(D) 
  /statetxt = first(statetxt)
  /DQSum=Sum(sameid)
  /DQN=n(sameid)
  /YearToUse=min(YearToUse)
  /Total_N=N.
DATASET ACTIVATE DQ_IDM.

Compute DQResult = DQSum/(DQN + Lag(DQN) - DQSum).

string DQVAR (a24).
compute DQVar = 'IDMatch'.
EXECUTE.

Select if subyr = YearToUse.
EXECUTE.
********************************************************************************************
* DQ Check: Child IDs for victims match across years, but dates of birth and sex do not match
********************************************************************************************.

DATASET ACTIVATE DQ_PerChild.

DO IF (subyr+1) = lag(subyr) AND chid = lag(chid).
    Compute agediff = lag(chage_max)-chage_min.

RECODE chsex_max (9=SYSMIS).
    DO IF (chage_min = 0).
        if ( (chbdate_max eq lag(chbdate_max)) and (chsex_max eq lag(chsex_max)) and (agediff ge 0 and agediff le 3)) notsamechild = 0.
        if ( (chbdate_max ne lag(chbdate_max)) OR (chsex_max ne lag(chsex_max)) OR (agediff lt 0 or agediff gt 3)) notsamechild = 1.       
    ELSE.
        if ((chsex_max eq lag(chsex_max)) and (agediff ge 0 and agediff le 3)) notsamechild = 0.
        if ((chsex_max ne lag(chsex_max)) OR (agediff lt 0 or agediff gt 3)) notsamechild = 1.       
    END IF.
END IF.
EXECUTE.

variable labels notsamechild  'Child with matching ids do not have the same sex and DOB'.
value labels notsamechild
0 'No'
1 'Yes'.

********************************************************************************************
* Save out individual child results of the same child DQ check for later use.
********************************************************************************************.
* Individual records will be excluded from the indicator calculations based on failing the same child check.

define !out_Ind() !quote(!concat(!unquote(!eval(!JOB_OUTPUTDATADIR)),"\IND_",
    !unquote(!eval(!JOB_DATAFILEROOT)),
    ".sav")) 
!enddefine.

TEMPORARY.
SELECT IF subyr = YearToUse.
  save outfile=!out_Ind /keep stateabb chid subyr notsamechild.

********************************************************************************************
* Aggregate and save out State level results of the same child DQ check.
********************************************************************************************.
*Aggregate.
DATASET DECLARE DQ_ChildM. 
AGGREGATE 
  /OUTFILE='DQ_ChildM' 
  /BREAK=stateabb(A) subyr(A)
  /statetxt = first(statetxt)
  /DQResult=Mean(notsamechild)
  /DQSum=sum(notsamechild)
  /DQN=n(notsamechild)
  /YearToUse=min(YearToUse)
  /Total_N=N.


DATASET ACTIVATE DQ_ChildM.

string DQVAR (a24).
compute DQVar = 'SameChild'.

Select if subyr = YearToUse.
EXECUTE.

********************************************************************************************
* Combine child level DQ files.
********************************************************************************************

DATASET DECLARE NCANDS_DQ. 
ADD FILES 
/file=DQ_ChildM
/file=DQ_IDM.

DATASET NAME NCANDS_DQ.
save outfile=!out.
DATASET CLOSE NCANDS_DQ.
