﻿* Encoding: UTF-8.

file handle output /name="Output".

* Open source data.  JOB_INPUTFILE is a file handle defined for each iteration
   by PROCESS FILES.
get file 'JOB_INPUTFILE'.
dataset name DQmain.

define !out() !quote(!concat(!unquote(!eval(!JOB_OUTPUTDATADIR)),"\",
    !unquote(!eval(!JOB_DATAFILEROOT)),
    ".sav")) 
!enddefine.

********************************************************************************************
* DQ Check: Missing age for victims.
********************************************************************************************.

DATASET ACTIVATE DQmain.

* Missing age for victims.
compute missingage = 1.
if (chage ge -1 and chage lt 99) missingage = 0.
variable labels missingage 'Child age is missing'.
value labels missingage
0 'No'
1 'Yes'.
EXECUTE.

DATASET DECLARE DQ_Age. 
AGGREGATE 
  /OUTFILE='DQ_Age' 
  /BREAK=stateabb(A) subyr(D) 
  /statetxt = first(statetxt)
  /DQSum=Sum(missingage)
  /DQN=n(missingage)
  /DQResult=MEAN(missingage).

DATASET ACTIVATE DQ_Age.

 string DQVAR (a24).
 compute DQVAR = 'MissingAge'.
EXECUTE.

********************************************************************************************
* DQ Check: Some victims should have AFCARS IDs in child file.
********************************************************************************************.

DATASET ACTIVATE DQmain.

compute hasafcars = 0.
if (afcarsid ne ' ') hasafcars = 1.
variable labels hasafcars 'AFCARS ID is reported'.
value labels hasafcars
0 'No'
1 'Yes'.
EXECUTE.

DATASET DECLARE DQ_AFCARS. 
AGGREGATE 
  /OUTFILE='DQ_AFCARS' 
  /BREAK=stateabb(A) subyr(D)
  /statetxt = first(statetxt) 
  /DQResult=MEAN(hasafcars)
  /DQSum=Sum(hasafcars)
  /DQN=n(hasafcars).

DATASET ACTIVATE DQ_AFCARS.

  string DQVAR (a24).
  compute DQVAR='HasAFCARSID'.
EXECUTE.

********************************************************************************************
* DQ Check: Some victims with AFCARS IDs should match IDs in AFCARS Files
********************************************************************************************.

DATASET ACTIVATE DQmain.


GET file='AFCARS_Comb_FH\CFSR 3 AFCARS source data.sav' 
/keep stateabb AFCARSID FY.
DATASET NAME AFCARS.
DATASET ACTIVATE AFCARS.

*Create a file with the list of unique ids (per year/state).
dataset declare UniqueAfcars.
aggregate outfile= UniqueAfcars
/break stateabb AFCARSID FY
/n1=n.

DATASET CLOSE AFCARS.


*********************************************************************************************************************************************************************.
* Combine the list of unique afcarsids with the NCANDS split file to run the AFCARS match test.
*********************************************************************************************************************************************************************.

DATASET ACTIVATE DQmain.

sort cases by stateabb afcarsid subyr.
execute.
match files 
   /file = DQmain
   /table =  UniqueAfcars
        /in=afcars
        /rename (FY=SubYR)    
   /by stateabb afcarsid SubYR.
execute.

if missing(AFCARSID) afcars = $sysmis. 
execute.

dataset declare AMmerge.
aggregate outfile=AMmerge
/break stateabb chid subyr
  /statetxt = first(statetxt)
/AFCARSID = MAX(AFCARSID)
/AFCARSMatch = max(afcars)
/NCANDSVics = N.
dataset activate AMmerge.

sort cases by stateabb subyr.
dataset declare DQ_stateAM.
aggregate outfile=DQ_stateAM
/break stateabb(A) subyr(D) 
/statetxt = first(statetxt)
/DQResult = mean(AFCARSMATCH)
/DQN=n(AFCARSMATCH)
/DQSum=Sum(AFCARSMATCH).

dataset activate DQ_stateAM.
  string DQVAR (a24).
  compute DQVAR = 'AFCARSMatch'.
execute.

********************************************************************************************
* Combine individual record level DQ files.
********************************************************************************************

DATASET DECLARE NCANDS_DQ. 
ADD FILES 
/file=DQ_Age
/file=DQ_AFCARS
/file DQ_stateAM.

DATASET NAME NCANDS_DQ.
save outfile=!out.
DATASET CLOSE NCANDS_DQ.
