﻿* Encoding: UTF-8.

****************************************************
CFSR 3 - CREATE AFCARS DQ FILES using previous splits output
The data quality file is also input to this code.  It should be in the Fixed Files
subdirectory of the work directory.
****************************************************.

* This syntax is applied to selected files as specified in the SPSSINC PROCESS FILES
   command.

* Open source data.  JOB_INPUTFILE is a file handle defined for each iteration
   by PROCESS FILES.

get file 'JOB_INPUTFILE'.
dataset name DQPeriod.

* The macro builds the output file specification using the macro definitions
   created by PROCESS FILES.

define !out() !quote(!concat(!unquote(!eval(!JOB_OUTPUTDATADIR)),"\",
    !unquote(!eval(!JOB_DATAFILEROOT)),
    ".sav")) 
!enddefine.


* Report states' data quality results for the 6-month period. 
****************************************************

* Send results to an SPSS file that will contain a history of the states' DQ performance for every 6-month period 
* examined. Includes the percentage of problem records and indicates whether a state failed or passed the 
* DQ limit for that check, for that 6-month period. A merged file containing all DQResults files will be called later 
* when running syntax for a specific indicator, to determine if a state should be excluded from the analysis.  


dataset declare DQResults.
aggregate 
  /OUTFILE='DQResults'
  /BREAK=state
  /stateabb=FIRST(stateabb)
  /DtReportBeg=FIRST(DtReportBeg)
  /DtReportEnd=FIRST(DtReportEnd)
  /DtReportEndFinal=FIRST(DtReportEndFinal)
  /SixMoPeriod=FIRST(SixMoPeriod)
  /GrandTotalRecords=N
  /DQ_Dropped=MEAN(DQ_Dropped)
  /DQ_IDNoMatchNext6Mo=MEAN(DQ_IDNoMatchNext6Mo)
  /DQ_missDOB =MEAN(DQ_missDOB)
  /DQ_missDtLatRem =MEAN(DQ_missDtLatRem)
  /DQ_missNumPlep =MEAN(DQ_missNumPlep)
  /DQ_DOBgtDtLatRem =MEAN(DQ_DOBgtDtLatRem)
  /DQ_DOBgtDtDisch =MEAN(DQ_DOBgtDtDisch)
  /DQ_gt21DOBtoDtLatRem =MEAN(DQ_gt21DOBtoDtLatRem)
  /DQ_gt21DOBtoDtDisch =MEAN(DQ_gt21DOBtoDtDisch)
  /DQ_gt21DtDischtoDtLatRem =MEAN(DQ_gt21DtDischtoDtLatRem)
  /DQ_DtDischeqDtLatRem =MEAN(DQ_DtDischeqDtLatRem)
  /DQ_DtDischltDtLatRem =MEAN(DQ_DtDischltDtLatRem)
  /DQ_missDisreasn =MEAN(DQ_missDisreasn)
  /DQ_totalrem1 =MEAN(DQ_totalrem1)
  /DQ_DroppedSum=SUM(DQ_Dropped)
  /DQ_IDNoMatchNext6MoSum=SUM(DQ_IDNoMatchNext6Mo)
  /DQ_missDOBSum=SUM(DQ_missDOB)
  /DQ_missDtLatRemSum=SUM(DQ_missDtLatRem)
  /DQ_missNumPlepSum=SUM(DQ_missNumPlep)
  /DQ_DOBgtDtLatRemSum=SUM(DQ_DOBgtDtLatRem)
  /DQ_DOBgtDtDischSum=SUM(DQ_DOBgtDtDisch)
  /DQ_gt21DOBtoDtLatRemSum=SUM(DQ_gt21DOBtoDtLatRem)
  /DQ_gt21DOBtoDtDischSum=SUM(DQ_gt21DOBtoDtDisch)
  /DQ_gt21DtDischtoDtLatRemSum=SUM(DQ_gt21DtDischtoDtLatRem)
  /DQ_DtDischeqDtLatRemSum=SUM(DQ_DtDischeqDtLatRem)
  /DQ_DtDischltDtLatRemSum=SUM(DQ_DtDischltDtLatRem)
  /DQ_missDisreasnSum=SUM(DQ_missDisreasn)
  /DQ_totalrem1Sum=SUM(DQ_totalrem1)
  /DQ_DroppedN=N(DQ_Dropped)
  /DQ_IDNoMatchNext6MoN=N(DQ_IDNoMatchNext6Mo)
  /DQ_missDOBN=N(DQ_missDOB)
  /DQ_missDtLatRemN=N(DQ_missDtLatRem)
  /DQ_missNumPlepN=N(DQ_missNumPlep)
  /DQ_DOBgtDtLatRemN=N(DQ_DOBgtDtLatRem)
  /DQ_DOBgtDtDischN=N(DQ_DOBgtDtDisch)
  /DQ_gt21DOBtoDtLatRemN=N(DQ_gt21DOBtoDtLatRem)
  /DQ_gt21DOBtoDtDischN=N(DQ_gt21DOBtoDtDisch)
  /DQ_gt21DtDischtoDtLatRemN=N(DQ_gt21DtDischtoDtLatRem)
  /DQ_DtDischeqDtLatRemN=N(DQ_DtDischeqDtLatRem)
  /DQ_DtDischltDtLatRemN=N(DQ_DtDischltDtLatRem)
  /DQ_missDisreasnN=N(DQ_missDisreasn)
  /DQ_totalrem1N=N(DQ_totalrem1).

dataset activate DQResults.

save outfile=!out.

varstocases
  /MAKE DQResult FROM DQ_Dropped to DQ_totalrem1
  /MAKE TotalRecords FROM DQ_DroppedN to DQ_totalrem1N
  /MAKE ProblemRecords FROM DQ_DroppedSUM to DQ_totalrem1SUM
  /INDEX=DQVar(DQResult) 
  /KEEP=state stateabb DtReportBeg DtReportEnd DtReportEndFinal SixMoPeriod GRANDTotalRecords
  /NULL=KEEP.

* If this is the last reporting period in the file, delete checks for DQ_Dropped and DQ_IDNoMatchNext6Mo.
* These require a subsequent reporting period.
do if DtReportEnd eq DtReportEndFinal.
select if not (DQVar = "DQ_Dropped" OR DQVar = "DQ_IDNoMatchNext6Mo").
end if.


sort cases by DQVar.

* Non active dataset is keyed table.
match files 
    /FILE=*  
    /TABLE='Fixed_FH\CFSR 3 AFCARS data quality checks.sav' 
    /BY DQVar. 


dataset activate DQResults.
if DQResult > DQLimit DQFailedCheck=1.
if sysmis(DQFailedCheck) DQFailedCheck=0.

save outfile=!out.


* Merge of all 6-month DQ results happens in AFTERSYNTAX step of PROCESS FILES.
